package editorListener.handlers;

import org.eclipse.core.filebuffers.FileBuffers;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import editorListener.listeners.FileBufferDispatch;
import editorListener.listeners.JavaElementDispatch;
import editorListener.listeners.PartDispatch;
import editorListener.listeners.PerspectiveDispatch;
import editorListener.listeners.WindowDispatch;

/**
 * This singleton class makes sure all of the services implemented are running.
 * It can be subclassed.
 * 
 * @author Matthew Harward
 * @version 3/8/2009
 */
public class ServiceChecker {

	protected boolean allServicesRunning;

	private static ServiceChecker sChecker;

	private ServiceChecker() {
		allServicesRunning = false;
	}

	public static ServiceChecker getInstance() {
		if (sChecker == null)
			sChecker = new ServiceChecker();
		return sChecker;
	}

	/**
	 * @return true if all services are running, otherwise false.
	 */
	public boolean servicesRunning() {
		return allServicesRunning;
	}

	/**
	 * This method attempts to start all required services.
	 */
	public void restartAllServices() {
		// Sets up the buffer listener.
		FileBuffers.getTextFileBufferManager().addFileBufferListener(
				FileBufferDispatch.getInstance());
		//JavaCore.addElementChangedListener(new JavaElementDispatch()); //???
		
		// Sets up a part listener
		if (PlatformUI.isWorkbenchRunning()) {
			PlatformUI.getWorkbench().addWindowListener(
					WindowDispatch.getInstance());
			for (IWorkbenchWindow ww : PlatformUI.getWorkbench()
					.getWorkbenchWindows()) {
				ww.addPerspectiveListener(new PerspectiveDispatch());
				for (IWorkbenchPage wp : ww.getPages()) {
					wp.addPartListener(new PartDispatch());
				}
			}
			allServicesRunning = true;
		} else
			allServicesRunning = false;
	}

	/**
	 * Clears services in use. This method makes no promise to clear all
	 * dispatch services in use.
	 */
	public void clearAllServices() {
		allServicesRunning = false;
		FileBuffers.getTextFileBufferManager().removeFileBufferListener(
				FileBufferDispatch.getInstance());
		if (PlatformUI.isWorkbenchRunning()) {
			PlatformUI.getWorkbench().removeWindowListener(
					WindowDispatch.getInstance());
		}
	}

	/**
	 * A convenience method for getting and starting all services.
	 */
	public static void start() {
		getInstance().restartAllServices();
	}
}
