package editorListener.handlers;

import org.eclipse.core.filebuffers.FileBuffers;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

/**
 * This data class holds references to all of the objects that represent the
 * updated object.
 * 
 * Note: This is a data class. It is felt that the decision to provide getters
 * is acceptable.
 * 
 * @author Matthew Harward
 * @version 3/8/2009
 */
public class UpdatedEditor implements Comparable<UpdatedEditor>{

	// Fields held
	protected IDocument theDocument;
	protected IWorkbenchPart thePart;
	protected EditorState theState;

	/**
	 * The primary constructor.
	 * 
	 * @param theDocument
	 *            the document representing the current editor's contents.
	 * @param thePart
	 *            the workbench part of the current editor.
	 * @param theState
	 *            the defined state of the current editor.
	 */
	public UpdatedEditor(IDocument theDocument, IWorkbenchPart thePart,
			EditorState theState) {
		this.theDocument = theDocument;
		this.thePart = thePart;
		this.theState = theState;
	}

	/**
	 * Gets the document representing the contents of the editor.
	 * 
	 * @return the document.
	 */
	public IDocument getDocument() {
		return theDocument;
	}

	/**
	 * Gets the part representing the editor in the eclipse workbench.
	 * 
	 * @return the editor.
	 */
	public IWorkbenchPart getWorkbenchPart() {
		return thePart;
	}

	/**
	 * Gets the state of the current editor.
	 * 
	 * @return the state.
	 */
	public EditorState getTheState() {
		return theState;
	}

	/**
	 * Gets the path of the current document.
	 * 
	 * @return the path
	 */
	public IPath getPath() {
		//return FileBuffers.createTextFileBufferManager().getTextFileBuffer(theDocument).
			//getLocation();
		return FileBuffers.getTextFileBufferManager().getTextFileBuffer(
				theDocument).getLocation();
	}

	/**
	 * Overrides the toString method.
	 */
	public String toString() {
		return "Editor Update - [ State: " + getTheState().toString()
				+ " Path: " + getPath().toString() + " ]";
	}

	@Override
	public int compareTo(UpdatedEditor o) {
		if(getPath().equals(o.getPath()) && 
				getWorkbenchPart().getTitle().equals(o.getWorkbenchPart()))
			return 1;
		else return -1;
	}
}
