package editorListener.listeners;

import org.eclipse.core.filebuffers.IFileBuffer;
import org.eclipse.core.filebuffers.IFileBufferListener;
import org.eclipse.core.filebuffers.ITextFileBuffer;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.text.IDocument;

import editorListener.handlers.EditorListener;

/**
 * This class listens to Eclipse's file buffer to check changes to the
 * underlying buffers to the editor.
 * 
 * @author Matthew Harward
 * @version 4/8/2009
 */
public class FileBufferDispatch extends Dispatch implements IFileBufferListener {

	private static FileBufferDispatch instance;

	protected FileBufferDispatch() {
		instance = this;
	}

	public static FileBufferDispatch getInstance() {
		if (instance == null) {
			instance = new FileBufferDispatch();
		}
		return instance;
	}

	private boolean checkType(IFileBuffer buff) throws CoreException {
		return EditorListener.getInstance().getTypeHandler().checkFileType(
				buff.getContentType().getName());
	}

	// Overridden Methods

	@Override
	public void bufferCreated(IFileBuffer buff) {
		try {
			if (buff instanceof ITextFileBuffer && checkType(buff)) {
				IDocument doc = ((ITextFileBuffer) buff).getDocument();
				EditorListener.getInstance().changeFocus(doc);
			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void dirtyStateChanged(IFileBuffer buff, boolean isDirty) {
		try {
			if (isDirty == true && buff instanceof ITextFileBuffer
					&& checkType(buff)) {
				//Using this buffer allows for more events to be called,
				//however some may be in the wrong place...
				//IDocument doc = ((ITextFileBuffer) buff).getDocument();
				//EditorListener.getInstance().updateCurrentDocument(doc);
			}
			if (isDirty == false && buff instanceof ITextFileBuffer
					&& checkType(buff)) {
				IDocument doc = ((ITextFileBuffer) buff).getDocument();
				EditorListener.getInstance().changeFocus(doc);
			}
		} catch (CoreException e) {
		}
	}

	// Unneeded Methods

	@Override
	public void stateChangeFailed(IFileBuffer buff) {
	}

	@Override
	public void stateChanging(IFileBuffer buff) {
	}

	@Override
	public void stateValidationChanged(IFileBuffer buff,
			boolean isStateValidated) {
	}

	@Override
	public void underlyingFileDeleted(IFileBuffer buff) {
	}

	@Override
	public void underlyingFileMoved(IFileBuffer buff, IPath path) {
	}

	@Override
	public void bufferContentAboutToBeReplaced(IFileBuffer buff) {
	}

	@Override
	public void bufferContentReplaced(IFileBuffer buff) {
		try {
			if (buff instanceof ITextFileBuffer && checkType(buff)) {
				IDocument doc = ((ITextFileBuffer) buff).getDocument();
				EditorListener.getInstance().changeFocus(doc);
			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void bufferDisposed(IFileBuffer buff) {
	}

}