package editorListener.listeners;

import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IWorkbenchPartReference;

import editorListener.handlers.EditorListener;

/**
 * This class listens to changes to parts, particularly the opening and closing
 * of tabs (which are parts) of the editor.
 * 
 * @author Matthew Harward
 * @version 4/8/2009
 */
@SuppressWarnings("restriction")
public class PartDispatch extends Dispatch implements IPartListener2 {

	@Override
	public void partClosed(IWorkbenchPartReference partRef) {
		if (EditorListener.getInstance().getTypeHandler().allowableObject(
				partRef.getPart(false))) {
			if (partRef.getPart(false) instanceof JavaEditor) {
				IDocument doc = ((JavaEditor) partRef.getPart(false))
						.getViewer().getDocument();
				EditorListener.getInstance().documentClosed(doc);
			}
		}

	}

	@Override
	public void partInputChanged(IWorkbenchPartReference partRef) {
		if (EditorListener.getInstance().getTypeHandler().allowableObject(
				partRef.getPart(false))) {
			if (partRef.getPart(false) instanceof JavaEditor) {
				IDocument doc = ((JavaEditor) partRef.getPart(false))
						.getViewer().getDocument();
				EditorListener.getInstance().updateCurrentDocument(doc);
			}
		}
	}

	@Override
	public void partOpened(IWorkbenchPartReference partRef) {
		if (EditorListener.getInstance().getTypeHandler().allowableObject(
				partRef.getPart(false))) {
			if (partRef.getPart(false) instanceof JavaEditor) {
				IDocument doc = ((JavaEditor) partRef.getPart(false))
						.getViewer().getDocument();
				EditorListener.getInstance().documentOpen(doc);
			}
		}
	}

	// Unnecessary Listeners

	@Override
	public void partDeactivated(IWorkbenchPartReference partRef) {
	}

	@Override
	public void partHidden(IWorkbenchPartReference partRef) {
	}

	@Override
	public void partVisible(IWorkbenchPartReference partRef) {
	}

	@Override
	public void partActivated(IWorkbenchPartReference partRef) {
		if (EditorListener.getInstance().getTypeHandler().allowableObject(
				partRef.getPart(false))) {
			if (partRef.getPart(false) instanceof JavaEditor) {
				IDocument doc = ((JavaEditor) partRef.getPart(false))
						.getViewer().getDocument();
				EditorListener.getInstance().documentOpen(doc);
			}
		}
	}

	@Override
	public void partBroughtToTop(IWorkbenchPartReference partRef) {
	}

}
