package metricsOverlay.api;

/**
 * This class provides a factory method that will provide an implementation
 * of <code>IMetricOverlay</code>.
 * 
 * It can be called in the following way:
 * <code>IMetricOverlay mOverlay = MetricOverlayFactory.instance();</code>
 * 
 * @author Matthew Harward
 * @version 1/7/2009
 */
public class MetricOverlayFactory {

	// The concrete implementation
	private static MetricOverlayAPI mOverlay;

	/**
	 * @return a concrete implementation of <code>IMetricOverlay</code>
	 */
	public static IMetricOverlay instance() {
		if (null == mOverlay) {
			createMetricOverlay();
		}
		return mOverlay;
	}

	/**
	 * A synchronised method that makes sure that the concrete
	 * implementation of <code>IMetricOverlay</code> is thread safe.
	 */
	private synchronized static final void createMetricOverlay() {
		if (null == mOverlay) {
			mOverlay = new MetricOverlayAPI();
		}
	}
}
