package metricsOverlay.augmentationStrategies;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.BackgroundLineAugmentation;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.swt.custom.LineBackgroundEvent;
import org.eclipse.swt.custom.LineBackgroundListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;

/**
 * This display strategy colours background lines of code with
 * a blend between two set colours.
 * 
 * @author Matthew Harward
 * @version 5/7/2009
 */
public class BgLineStrategy extends AugmentationStrategy {

	//private TwoColorManager cManager;
	private ModifyListener mListener;
	private LineBackgroundListener lineListener;

	public BgLineStrategy() {
		//cManager = new TwoColorManager();
	}
/*
	public TwoColorManager getColorManager() {
		return cManager;
	}

	@Override
	public int getAugMax() {
		return cManager.getRange();
	}

	@Override
	public int getAugMin() {
		return 0;
	}*/
	
	/**
	 * Apply properties to the display strategy
	 *//*
	protected void applyProperties(Map<String, String> prop) {
		SortedMap<Integer,Color> cMap = new TreeMap<Integer,Color>();
		Iterator<String> it = prop.keySet().iterator();
		while (it.hasNext()){
			String s = it.next();
			String value = prop.get(s);
			if(s.startsWith(MappingProperties.COLOR)){
				int i = Integer.parseInt(s.substring(MappingProperties.COLOR.length()));
				Color c = parseColor(value);
				cMap.put(i, c);
			}	
		}
		if(cMap.size() > 1) {
		Iterator<Integer> itint = cMap.keySet().iterator();
		cManager.setLow(cMap.get(itint.next()));
		cManager.setHigh(cMap.get(itint.next()));
		}
	}*/

	/**
	 * Adds line listeners for colouring the background and updating when the
	 * text changes.
	 */
	@Override
	public void update(MetricFile mf, Mapping mm) {
		super.update(mf, mm);
		// Adds the appropriate colour when a line is updated
		lineListener = new LineBackgroundListener() {

			public void lineGetBackground(final LineBackgroundEvent event) {
					Mapping curMap = mapping;
						for (MetricSection mSec : curMap.getMetric().getMetricSections()) {
							int curLine = textWidget
									.getLineAtOffset(event.lineOffset);
							if (mSec.getStartLine() <= curLine
									&& mSec.getEndLine() >= curLine) {
								int a = curMap.getInterpolatedValue(mSec
										.getValue());
								event.lineBackground = ((BackgroundLineAugmentation)curMap.getAugmentation())
								.getCManager().getColor(a);
					}
				}
			}
		};
		textWidget.addLineBackgroundListener(lineListener);

		// If the text is modified, make sure a redraw is fired.
		mListener = new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				textWidget.getDisplay().asyncExec(new Runnable() {
					@Override
					public void run() {
						textWidget.redraw();
					}
				});
			}
		};
		textWidget.addModifyListener(mListener);
	}

	@Override
	public void dispose() {
		textWidget.removeModifyListener(mListener);
		textWidget.removeLineBackgroundListener(lineListener);
		textWidget.redraw();
	}
	
}
