package metricsOverlay.augmentationStrategies;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;

/**
 * This display strategy colours background lines of code with a blend between
 * two set colours.
 * 
 * @author Matthew Harward
 * @version 5/7/2009
 */
public class BgSectionStrategy extends AugmentationStrategy {

	//private TwoColorManager cManager;
	private ModifyListener mListener;

	public BgSectionStrategy() {
		//cManager = new TwoColorManager();
	}
/*
	public TwoColorManager getColorManager() {
		return cManager;
	}

	@Override
	public int getAugMax() {
		return cManager.getRange();
	}

	@Override
	public int getAugMin() {
		return 0;
	}*/

	/**
	 * Apply properties to the display strategy
	 */
	/*protected void applyProperties(Map<String, String> prop) {
		SortedMap<Integer, Color> cMap = new TreeMap<Integer, Color>();
		Iterator<String> it = prop.keySet().iterator();
		while (it.hasNext()) {
			String s = it.next();
			String value = prop.get(s);
			if (s.startsWith(MappingProperties.COLOR)) {
				int i = Integer.parseInt(s.substring(MappingProperties.COLOR
						.length()));
				Color c = parseColor(value);
				cMap.put(i, c);
			}
		}
		if (cMap.size() > 1) {
			Iterator<Integer> itint = cMap.keySet().iterator();
			cManager.setLow(cMap.get(itint.next()));
			cManager.setHigh(cMap.get(itint.next()));
		}
	}*/

	/**
	 * Adds line listeners for colouring the background and updating when the
	 * text changes.
	 */
	@Override
	public void update(MetricFile mf, Mapping mm) {
		super.update(mf, mm);
		// Adds the appropriate colour when a line is updated

		/**
		 * textWidget.addLineStyleListener(new LineStyleListener() {
		 * 
		 * @Override public void lineGetStyle(LineStyleEvent event) {
		 *           Set<StyleRange> styRangeSet = new HashSet<StyleRange>();
		 *           StyleRange sr = new StyleRange(30,35, null, new
		 *           Color(null,250,125,10)); styRangeSet.add(sr); StyleRange[]
		 *           srArr = event.styles; if(srArr != null)
		 *           styRangeSet.addAll(Arrays.asList(srArr)); event.styles =
		 *           new StyleRange[styRangeSet.size()];
		 *           styRangeSet.toArray(event.styles); } });
		 */

		// If the text is modified, make sure a redraw is fired.
		mListener = new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				textWidget.getDisplay().asyncExec(new Runnable() {
					@Override
					public void run() {
						Mapping curMap = mapping;
						for (MetricSection mSec : curMap.getMetric().getMetricSections()) {
							StyleRange[] srArr = textWidget.getStyleRanges(getOffset(mSec),
									getLength(mSec));
							Set<StyleRange> styRangeSet = new HashSet<StyleRange>();
							if (srArr != null)
								styRangeSet.addAll(Arrays.asList(srArr));
							int a = curMap.getInterpolatedValue(mSec.getValue());
							Color curColor = curMap.getAugmentation().getCManager().getColor(a);
							for (StyleRange range : styRangeSet) {
								range.background = curColor;
								textWidget.setStyleRange(range);
							}
						}
						textWidget.redraw();
					}
				});
			}
		};
		textWidget.addModifyListener(mListener);
	}
	
	@Override
	public void dispose() {
		textWidget.removeModifyListener(mListener);
		textWidget.redraw();
	}

}
