package metricsOverlay.augmentationStrategies;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.jface.text.IViewportListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;

/**
 * This class implements colour chips as a design strategy.
 * 
 * @author Matthew Harward
 * @version 6/7/2009
 *
 */
public class MarginBarStrategy extends AugmentationStrategy {

	private static final int OFFSET = 1;
	protected static final int BARWIDTH = 30;
	// Fields
	//private TwoColorManager cManager1;
	//private TwoColorManager cManager2;
	
	private int verticalOffset; // Default - 0
	private ModifyListener mListener;
	private IViewportListener vListener;
	private PaintListener paintListener;
	
	public MarginBarStrategy() {
		//cManager1 = new TwoColorManager();
		//cManager2 = new TwoColorManager();
		verticalOffset = 0;
	}
	/**
	public TwoColorManager getColorManagerL() {
		return cManager1;
	}
	
	public TwoColorManager getColorManagerR() {
		return cManager2;
	}


	@Override
	public int getAugMax() {
		return cManager1.getRange() + cManager2.getRange(); //xxx
	}

	@Override
	public int getAugMin() {
		return 0;
	}*/
	

	/**
	 * Apply properties to the display strategy
	 *//*
	protected void applyProperties(Map<String, String> prop) {
		SortedMap<Integer,Color> cMap = new TreeMap<Integer,Color>();
		Iterator<String> it = prop.keySet().iterator();
		while (it.hasNext()){
			String s = it.next();
			String value = prop.get(s);
			if(s.startsWith(MappingProperties.COLOR)){
				int i = Integer.parseInt(s.substring(MappingProperties.COLOR.length()));
				Color c = parseColor(value);
				cMap.put(i, c);
			}	
		}
		if(cMap.size() > 3) {
		Iterator<Integer> itint = cMap.keySet().iterator();
		cManager1.setLow(cMap.get(itint.next()));
		cManager1.setHigh(cMap.get(itint.next()));
		cManager2.setLow(cMap.get(itint.next()));
		cManager2.setHigh(cMap.get(itint.next()));
		}
	}*/

	@Override
	public void update(MetricFile mf, Mapping mm) {
		super.update(mf, mm);
		
		paintListener = new PaintListener() {
			@Override
			public void paintControl(PaintEvent e) {
				textWidget.setIndent(BARWIDTH + OFFSET*2);
				// Set constants
				int lineHeight = textWidget.getLineHeight();
				// Retain original bg
				Color orgBColor = e.gc.getBackground();
				Color orgFColor = e.gc.getForeground();
				// Add chips as needed
				//TODO
				//TODO
				for (Mapping curMap : curMFile.getMetricDocument()
						.getAllMappings()) {
						for (MetricSection mSec : curMap.getMetric().getMetricSections()) {
							for (int j = mSec.getStartLine(); j <= mSec
									.getEndLine(); j++) {
								int a = curMap.getInterpolatedValue(mSec
										.getValue());
								//Need two different pieces of metric info for this mapping
								//TODO 
								e.gc.setBackground(new Color(null,120,120,120));//cManager2.getColor(a));
								e.gc.setForeground(new Color(null,250,120,120));//cManager1.getColor(a));
								e.gc.fillGradientRectangle(OFFSET, yPos(j), BARWIDTH/*+100*/, lineHeight - 2, false);
								//e.gc.setBackgroundPattern(new Pat)
								//e.gc.setAlpha(100);
							}
					}
				}
				// Reset bg color
				e.gc.setBackground(orgBColor);
				e.gc.setForeground(orgFColor);
			}

		};
		textWidget.addPaintListener(paintListener);

		vListener = new IViewportListener() {
			@Override
			public void viewportChanged(int vOffset) {
				verticalOffset = vOffset;
				textWidget.redraw();

			}
		};
		viewer.addViewportListener(vListener);

		mListener = new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				textWidget.getDisplay().asyncExec(new Runnable() {
					@Override
					public void run() {
						textWidget.redraw();
						textWidget.getDisplay().update();
					}
				});
			}
		};
		textWidget.addModifyListener(mListener);
	}

	private int yPos(int lineNum) {
		int lineHeight = textWidget.getLineHeight();
		return lineNum * lineHeight - verticalOffset + 2;
	}
	
	@Override
	public void dispose() {
		textWidget.removeModifyListener(mListener);
		textWidget.removePaintListener(paintListener);
		viewer.removeViewportListener(vListener);
		viewer.refresh();
		textWidget.redraw();
	}
}
