package metricsOverlay.augmentationStrategies;

import java.util.Set;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.jface.text.IViewportListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

public class MarginChipStrategy extends AugmentationStrategy {

		// Constants
		private static final int CHIPWIDTH = 14;
		private static final int CHIPXSPACE = 2;
		private static final int CHIPOFFSET = 1;
		private static final int SCROLLWIDTH = 5;
		private static final int ARC = 3;
		// Fields
		//private IColorManager cManager;
		//private int verticalOffset; // Default - 0
		//private boolean left; // Default - left
		//private int numberOfChips; // Default - 1
		//private int position; // Default - 0
		// The shape
		//private String shapeName;
		private Set<int[]> shape;
		private ModifyListener mListener;
		private IViewportListener vListener;
		private PaintListener paintListener;
		private MouseMoveListener mMoveListener;

		public MarginChipStrategy() {
			//cManager = new TwoColorManager();
			//verticalOffset = 0;
			//setChipSide(true);
			//setNumberOfChips(1);
			//setPosition(0);
		}

		/**
		 * Sets the number of chips on the specified side.
		 * 
		 * @param numberOfChips
		 */
		/*public void setNumberOfChips(int numberOfChips) {
			this.numberOfChips = numberOfChips;
		}

		public int getNumberOfChips() {
			return numberOfChips;
		}

		public void setPosition(int position) {
			this.position = position;
		}

		public int getPosition() {
			return position;
		}

		public TwoColorManager getColorManager() {
			return cManager;
		}

		public void setChipSide(boolean left) {
			this.left = left;
		}

		public boolean isChipOnLeftSide() {
			return left;
		}

		@Override
		public int getAugMax() {
			return cManager.getRange();
		}

		@Override
		public int getAugMin() {
			return 0;
		}*/

		/**
		 * Apply properties to the display strategy
		 *//*
		protected void applyProperties(Map<String, String> prop) {
			SortedMap<Integer, Color> cMap = new TreeMap<Integer, Color>();
			Iterator<String> it = prop.keySet().iterator();
			while (it.hasNext()) {
				String s = it.next();
				String value = prop.get(s);
				if (s.startsWith(MappingProperties.COLOR)) {
					int i = Integer.parseInt(s.substring(MappingProperties.COLOR
							.length()));
					Color c = parseColor(value);
					cMap.put(i, c);
				} else if (s.equalsIgnoreCase(MappingProperties.ONLEFT)) {
					left = Boolean.parseBoolean(value);
				} else if (s.equalsIgnoreCase(MappingProperties.NUMCHIPS)) {
					numberOfChips = Integer.parseInt(value);
				} else if (s.equalsIgnoreCase(MappingProperties.CHIPPOSITION)) {
					position = Integer.parseInt(value);
				} else if (s.equalsIgnoreCase(MappingProperties.SHAPE)) {
					shapeName = value;
					shape = ShapeManager
							.getShape(textWidget.getLineHeight(), value);
				}
			}
			if (cMap.size() > 1) {
				Iterator<Integer> itint = cMap.keySet().iterator();
				cManager.setLow(cMap.get(itint.next()));
				cManager.setHigh(cMap.get(itint.next()));
			}
		}*/

		@Override
		public void update(MetricFile mf, final Mapping mm) {
			super.update(mf, mm);
			/*
			ProjectionAnnotationModel pam = viewer.getProjectionAnnotationModel();
			IAnnotationModel iam = viewer.getVisualAnnotationModel();
			iam.addAnnotation(new Annotation("", true,"HelloWorld"), new Position(0,10));
			iam.connect(curMFile.getDocument()); */
			
			paintListener = new PaintListener() {
				@Override
				public void paintControl(PaintEvent e) {
					MarginAugmentation ma = (MarginAugmentation)mapping.getAugmentation();
					// Set margins as needed
					if (ma.isLeft())
						textWidget.setIndent(CHIPOFFSET * 2
								+ (CHIPWIDTH + CHIPXSPACE) * ma.totChipsOnSide());
					else {
						// A right margin currently does not work, right side colour
						// chips will obscure text...
						// textWidget.setRightMargin(CHIPOFFSET*2 + (CHIPWIDTH +
						// CHIPXSPACE)*numberOfChips + SCROLLWIDTH);
					}

					// Set constants
					int lineHeight = textWidget.getLineHeight();
					Point size = textWidget.getSize();
					int width = size.x;
					//Get shape
					shape = ma.getShape(lineHeight);
					// Retain original bg
					Color orgColor = e.gc.getBackground();
					// Add chips as needed
					Mapping curMap = mapping;
					for (MetricSection mSec : curMap.getMetric().getMetricSections()) {
						for (int j = mSec.getStartLine(); j <= mSec.getEndLine(); j++) {
							int a = curMap.getInterpolatedValue(mSec.getValue());
							e.gc.setBackground(ma.getCManager().getColor(a));
							if (shape != null){
								for (int[] arr : shape) {
									int[] finalShape = new int[arr.length];
									for (int i = 0; i < arr.length; i++) {
										if (i % 2 == 0)
											finalShape[i] = arr[i] + xPos(width, ma);
										else
											finalShape[i] = arr[i] + yPos(j);
									}
									e.gc.fillPolygon(finalShape);
								}
							}
							else if (ma.getShapeType().equals(ShapeType.circle)){
								e.gc.fillOval(xPos(width,ma), yPos(j), lineHeight - 2,
										lineHeight - 2);
							} else {
								e.gc.fillRoundRectangle(xPos(width,ma), yPos(j),
										CHIPWIDTH, lineHeight - 2, ARC, ARC);
							}
							//if(mSec.hasAnnotation()) {
							//	Rectangle rec = new Rectangle(xPos(width,ma), yPos(j),CHIPWIDTH,lineHeight - 2);
							//	addToolTip(mSec,rec);
							//}
						}
					}
					// Reset bg color
					e.gc.setBackground(orgColor);
				}


			};
			textWidget.addPaintListener(paintListener);
			
			vListener = new IViewportListener() {
				@Override
				public void viewportChanged(int vOffset) {
					//verticalOffset = vOffset;
					textWidget.update();
				}
			};
			viewer.addViewportListener(vListener);

			mListener = new ModifyListener() {
				@Override
				public void modifyText(ModifyEvent e) {
					textWidget.getDisplay().asyncExec(new Runnable() {
						@Override
						public void run() {
							textWidget.redraw();
							textWidget.getDisplay().update();
						}
					});
				}
			};
			textWidget.addModifyListener(mListener);
			
			mMoveListener = new MouseMoveListener() {
				
				@Override
				public void mouseMove(MouseEvent e) {
					//If it is in the right position
					int index = textWidget.getLineIndex(e.y);
					int pos = xPos(textWidget.getSize().x,
							(MarginAugmentation) mm.getAugmentation());
					if(e.x > pos && e.x <= (pos + CHIPWIDTH))
					{
						//See if we have an appropriate augmentation.
						for(MetricSection mSec : mm.getMetric().getMetricSections()){
							if(mSec.hasAnnotation() && index >= mSec.getStartLine() &&
									index <= mSec.getEndLine()){
								String text = mSec.getAnnotation();
								if (!(text.equals(textWidget.getToolTipText ()))) {
									textWidget.setToolTipText (text);
								}
							}
								
						}
					} else textWidget.setToolTipText("");
				}
			};
			textWidget.addMouseMoveListener(mMoveListener);
			
		}

		private int xPos(int editorWidth, MarginAugmentation ma) {
			if (ma.isLeft()) {
				return CHIPOFFSET + ma.getMarginPosition() * (CHIPWIDTH + CHIPXSPACE);
			} else {
				return editorWidth - SCROLLWIDTH - (ma.totChipsOnSide() - ma.getMarginPosition() + 1)
						* (CHIPWIDTH + CHIPXSPACE);
			}
		}

		private int yPos(int lineNum) {
			//int lineHeight = textWidget.getLineHeight();
			//return lineNum * lineHeight - verticalOffset + 2;
			return textWidget.getLinePixel(lineNum);
		}
		
		@Override
		public void dispose() {
			textWidget.removeModifyListener(mListener);
			textWidget.removePaintListener(paintListener);
			viewer.removeViewportListener(vListener);
			textWidget.update();
			textWidget.redraw();
		}

	
}
