package metricsOverlay.colorManagers;

import org.eclipse.swt.graphics.Color;

public class TwoColorManager implements IColorManager{

	private Color low;
	private Color high;

	public TwoColorManager(Color low, Color high) {
		this.low = low;
		this.high = high;
	}
	
	/**
	 * This default constructor sets all colours to white.
	 */
	public TwoColorManager() {
		this(new Color(null,255,255,255), new Color(null,255,255,255));
	}

	public int getRange() {
		int sum = 0;
		sum += Math.abs(high.getRed() - low.getRed());
		sum += Math.abs(high.getBlue() - low.getBlue());
		sum += Math.abs(high.getGreen() - low.getGreen());
		return sum;
	}
	
	/**
	 * Ratio method of interpolation
	 */
	public Color getColor2(int value) {
		float val = value;
		float range = getRange();
		float ratio = 1 - (val/range);
		float lowRed = low.getRed() < high.getRed() ? low.getRed() : high.getRed(); 
		float lowGreen = low.getGreen() < high.getGreen() ? low.getGreen() : high.getGreen();
		float lowBlue = low.getBlue() < high.getBlue() ? low.getBlue() : high.getBlue();
		Float red = lowRed +  (Math.abs(high.getRed()-low.getRed())*ratio);
		Float green = lowGreen +  (Math.abs(high.getGreen()-low.getGreen())*ratio);
		Float blue = lowBlue +  (Math.abs(high.getBlue()-low.getBlue())*ratio);
		//System.out.println("Color: " + red.intValue() + ", " +
			//	green.intValue()+ ", " + blue.intValue() + "from ratio: " + ratio);
		return new Color(null, red.intValue(), 
				green.intValue(), blue.intValue());
	}
	
	/**
	 * Using HSV, not working...
	 */
	public Color getColor3(int value) {
		float val = value;
		float range = getRange();
		float ratio = 1 - (val/range);
		float[] hsvLow = new float[3];
		float[] hsvHigh = new float[3];
		ColorUtils.convertRGBtoHSV(low.getRed(), low.getGreen(), 
				low.getBlue(), hsvLow);
		ColorUtils.convertRGBtoHSV(high.getRed(), high.getGreen(), 
				high.getBlue(), hsvHigh);
		//System.out.println(hsvHigh[0] +", " + hsvHigh[1] + ", " + hsvHigh[2]);
		float lowH = hsvLow[0] < hsvHigh[0] ? hsvLow[0] : hsvHigh[0]; 
		float lowS = hsvLow[2] < hsvHigh[2] ? hsvLow[2] : hsvHigh[2]; 
		float lowV = hsvLow[2] < hsvHigh[2] ? hsvLow[2] : hsvHigh[2]; 
		Float h = lowH + Math.abs(hsvLow[0] - hsvHigh[0])*ratio;
		Float s = lowS + Math.abs(hsvLow[1] - hsvHigh[1])*ratio;
		Float v = lowV + Math.abs(hsvLow[2] - hsvHigh[2])*ratio;
		float[] rgb = new float[3];
		ColorUtils.convertHSVtoRGB(h, s, v, rgb);
		//System.out.println(rgb[0] +", " + rgb[1] + ", " + rgb[2]);
		return new Color(null, Float.valueOf(rgb[0]).intValue(), 
				Float.valueOf(rgb[1]).intValue(), 
				Float.valueOf(rgb[2]).intValue());
	}

	/**
	 * Straight linear interpolation
	 */
	public Color getColor(int value) {
		if (value <= 0)
			return low;
		else if (value >= getRange())
			return high;
		else {
			int count = 0;
			int red, redHigh, green, greenHigh, blue, blueHigh = 0;
			//Red
				red = low.getRed();
				redHigh = high.getRed();
			
			//Green
				green = low.getGreen();
				greenHigh = high.getGreen();
			//Blue
				blue = low.getBlue();
				blueHigh = high.getBlue();
			//Iterate through
			while (true) {
				if (red < redHigh) {
					red++;
					count++;
					if (count >= value)
						break;
				} else {
					red--;
					count++;
					if (count >= value)
						break;
				}
				if (green < greenHigh) {
					green++;
					count++;
					if (count >= value)
						break;
				} else {
					green--;
					count++;
					if (count >= value)
						break;
				}
				if (blue < blueHigh) {
					blue++;
					count++;
					if (count >= value)
						break;
				} else {
					blue--;
					count++;
					if (count >= value)
						break;
				}
			}
			return new Color(null, red, green, blue);
		}
	}

	public Color getLow() {
		return low;
	}

	public void setLow(Color low) {
		this.low = low;
	}

	public Color getHigh() {
		return high;
	}

	public void setHigh(Color high) {
		this.high = high;
	}

	@Override
	public int getNumColors() {
		return 2;
	}

}
