package metricsOverlay.editor;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;

import metricsOverlay.Activator;
import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentInformationMapping;
import org.eclipse.jface.text.ISlaveDocumentManager;
import org.eclipse.jface.text.ISlaveDocumentManagerExtension;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.text.projection.ProjectionDocument;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModelEvent;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModelListener;
import org.eclipse.jface.text.source.IAnnotationModelListenerExtension;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.swt.custom.LineBackgroundEvent;
import org.eclipse.swt.custom.LineBackgroundListener;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.AbstractDecoratedTextEditorPreferenceConstants;
import org.eclipse.ui.texteditor.AbstractTextEditor;

@SuppressWarnings("restriction")
public class DecoratorEditor {
	private static final int NONE = 0;
	private static final int TIME = 1;
	private static final int LINEAR = 1;
	private static final int EXPOTENTIAL = 2;
	private static final int EARLIESTDATE = 1;
	private static final int SETDATES = 2;
	private static final int OPENTIME = 3;
	private static final int SETSTARTDATE = 4;
	private static final int SETENDDATE = 5;
	private CompilationUnitEditor editor = null;
	private ProjectionViewer viewer = null;
	private StyledText textWidget = null;
	private int coloringType = NONE;
	private int equationType = NONE;
	private boolean hasFocus = true;
	private boolean colour = false;
	private Color startColor = null;
	private Color endColor = null;
	private int dateType = OPENTIME;
	private long beginTime = 0;
	private long endTime = 0;
	private MetricDocument mapping;

	public DecoratorEditor(CompilationUnitEditor editor) {
		this.editor = editor;
		System.out.println("Constructor Called!");
		try {
			Method getSourceViewer = AbstractTextEditor.class
					.getDeclaredMethod("getSourceViewer");
			getSourceViewer.setAccessible(true);
			ProjectionViewer viewer = (ProjectionViewer) getSourceViewer
					.invoke(editor);
			this.viewer = viewer;
			textWidget = viewer.getTextWidget();
			// textWidget.set
			colour = true;
			// setColoring(true);
			addLineListener();
			textWidget.redraw();
			// Add the ChangeTracker which will record the time user and class
			// which changes are being made to.
			// TODO //viewer.getDocument().addDocumentListener(new
			// ChangeTracker(editor));
			// Add some make believe annotations for testing purposes.
		} catch (Exception e) {
			System.err.println("Unable to add LineBackgroundListener");
			e.printStackTrace();
		}
	}

	public DecoratorEditor(CompilationUnitEditor doc, MetricDocument mapping) {
		this(doc);
		this.mapping = mapping;
		System.out.println("Call the editor!");

		// TODO Auto-generated constructor stub
	}

	public void addLineListener() {

		textWidget.addLineBackgroundListener(new LineBackgroundListener() {

			public void lineGetBackground(final LineBackgroundEvent event) {
				textWidget.getDisplay().syncExec(new Runnable() {
					public void run() {
						System.out.println("Background Event Called" + colour);
						// int val = event.lineOffset;
						// if(val > 255) val = 255;
						// if (colour) event.lineBackground = new
						// Color(null,val,155,166);//getColorByOffset(event);
						for (Mapping curMap : mapping.getAllMappings()) {
							for (MetricSection mSec : curMap.getMetric().getMetricSections()) {
								int a = curMap.getInterpolatedValue(mSec
										.getValue());
								if (colour
										&& event.lineOffset >= mSec
												.getStartLine()
										&& event.lineOffset <= mSec
												.getEndLine())
									event.lineBackground = new Color(null, a,
											a, a);
							}
						}
						// textWidget.redraw();
					}
				});
			}
		});
/**
		viewer.getTextWidget().addDisposeListener(new DisposeListener() {

			@Override
			public void widgetDisposed(DisposeEvent event) {
				LineHighlighterManager highlighterManager = LineHighlighterManager
						.getInstance();
				highlighterManager.removeHighLighter(editor);
			}

		});*/
	}

	/*
	 * Gets an IDocumentInformationMapping for the master IDocument to the
	 * ProjectionDocument.
	 * 
	 * @author Colin Fagg and Linda Pettigrew
	 * 
	 * @date 23 April 2009
	 */
	private IDocumentInformationMapping getProjectViewerMap() {
		IDocumentInformationMapping map = null;
		ISlaveDocumentManagerExtension docManager = null;

		try {
			// Gain access to the protected method
			// getMasterSlaveDocumentManager.
			Method getSlaveDocumentManager = TextViewer.class
					.getDeclaredMethod("getSlaveDocumentManager");
			getSlaveDocumentManager.setAccessible(true);
			docManager = (ISlaveDocumentManagerExtension) getSlaveDocumentManager
					.invoke(viewer);

			IDocument[] slaveDocs = docManager.getSlaveDocuments(viewer
					.getDocument());
			ProjectionDocument pDoc = null;

			// Finding the Slave Document which is a ProjectionDocument.
			for (int i = 0; i < slaveDocs.length; i++) {
				if (slaveDocs[i].getClass().equals(ProjectionDocument.class)) {
					pDoc = (ProjectionDocument) slaveDocs[i];
					break;
				}
			}
			// This is the mapping between the slave document and the master.
			map = ((ISlaveDocumentManager) docManager)
					.createMasterSlaveMapping(pDoc);
		} catch (Exception e) {

		}
		return map;
	}

	/*
	 * Returns the colour for the event for this line based on line offset and
	 * the LineHighlighters colouringType
	 * 
	 * @author Colin Fagg and Linda Pettigrew
	 * 
	 * @date 23 April 2009
	 */
	private Color getColorByOffset(LineBackgroundEvent event) {
		if (event.lineText.matches("[\\s]*")) {
			return null;
		}
		updatePreferences();
		IAnnotationModel model = viewer.getAnnotationModel();
		Color color = null;
		IDocumentInformationMapping map = getProjectViewerMap();
		try {
			Iterator it = model.getAnnotationIterator();
			Annotation current = null;

			if (coloringType == NONE) {
				return color;
			}

			while (it.hasNext()) {
				// System.out.println("in While loop1");
				current = (Annotation) it.next();
				if (current.getType().matches("Colouring")) {
					if (map == null) {
						if (model.getPosition(current).includes(
								event.lineOffset)) {
							current.getText();
							color = getColorByColouring(current.getText());// new
							// Color(null,
							// 255,255,25);
							break;
						}
					} else if (model.getPosition(current).includes(
							map.toOriginOffset(event.lineOffset))) {
						current.getText();
						color = getColorByColouring(current.getText());// new
						// Color(null,
						// 255,255,25);
						break;
					}

				}
			}
		} catch (Exception e) {
			System.err.println("error in getColorByOffset:");
			e.printStackTrace();
		}
		return color;
	}

	private void updatePreferences() {
		// TODO Auto-generated method stub
		setEquationType();
		setColoringType();
		setStartColor();
		setEndColor();
		setBeginTime();
		setEndTime();
	}

	private void setColoringType() {
		// TODO stub currently returns generic choice
		coloringType = TIME;
	}

	private void setEquationType() {
		// TODO stub currently returns generic choice
		equationType = EXPOTENTIAL;
	}

	private void setEndTime() {
		// TODO stub currently returns generic choice
		switch (dateType) {
		case EARLIESTDATE:
		case OPENTIME:
		case SETSTARTDATE:
			endTime = (new java.util.Date()).getTime();
			break;
		case SETDATES:
		case SETENDDATE:
			endTime = getEndTime();
			break;
		case NONE:
			break;
		}
	}

	private void setBeginTime() {
		// TODO stub currently returns generic choice
		switch (dateType) {
		case EARLIESTDATE:
			if (beginTime == 0) {
				beginTime = getEarliestTimeInDoc();
			}
		case OPENTIME:
		case SETENDDATE:
			if (beginTime == 0) {
				beginTime = (new java.util.Date()).getTime();
			}
			break;
		case SETDATES:
		case SETSTARTDATE:
			beginTime = getBeginTime();
			break;
		case NONE:
			break;
		}
	}

	private long getEarliestTimeInDoc() {
		// TODO Auto-generated method stub
		Iterator it = viewer.getAnnotationModel().getAnnotationIterator();
		Annotation current = null;
		long smallestVal = (new java.util.Date()).getTime();
		long tempVal = smallestVal;
		while (it.hasNext()) {
			current = (Annotation) it.next();
			if (current.getType().matches("Colouring")) {
				tempVal = parseLongFromString("Time", current.getText());
				if (tempVal < smallestVal) {
					smallestVal = tempVal;
				}
			}
		}
		return smallestVal;
	}

	private void setEndColor() {
		// TODO stub currently returns generic choice
		endColor = getEndColor();// new Color(null, 0,255, 255);
	}

	private void setStartColor() {
		// TODO stub currently returns generic choice
		startColor = getStartColor();// new Color(null, 200,255, 255);
	}

	/*
	 * Returns the colour based on the annotation text and coloringType
	 * (accesses appropriate colouring algorithm)
	 * 
	 * @author Colin Fagg and Linda Pettigrew
	 * 
	 * @date 23 April 2009
	 */
	private Color getColorByColouring(String text) {
		Color color = null;
		double relativePosition = -1;
		if (coloringType == TIME) {
			long value = parseLongFromString("Time", text);
			// System.out.println("datevalue:" + text);
			relativePosition = calcRelativeTimeColor(value);
		} else {
			color = null;
		}
		if ((relativePosition <= 1) && (relativePosition >= 0)) {
			if (equationType == LINEAR) {
				color = getColorByEquation(relativePosition, "linearRange");
			} else if (equationType == EXPOTENTIAL) {
				color = getColorByEquation(relativePosition, "expotentialRange");
			}
		}
		return color;
	}

	private Color getColorByEquation(double relativeLocation,
			String equationName) {
		try {
			Method equation = DecoratorEditor.class.getDeclaredMethod(
					equationName, new Class[] { Double.TYPE, Double.TYPE,
							Double.TYPE });
			return new Color(null, ((Integer) equation.invoke(this,
					new Object[] { endColor.getRed(), startColor.getRed(),
							relativeLocation })).intValue(),
					((Integer) equation.invoke(this, new Object[] {
							endColor.getGreen(), startColor.getGreen(),
							relativeLocation })).intValue(),
					((Integer) equation.invoke(this, new Object[] {
							endColor.getBlue(), startColor.getBlue(),
							relativeLocation })).intValue());
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	protected int linearRange(double endCol, double startCol, double position) {
		return (int) ((endCol - startCol) * position + startCol);
	}

	protected int expotentialRange(double endCol, double startCol,
			double position) {
		return (int) ((endCol - startCol) * java.lang.Math.exp(position - 1) + startCol);
	}

	@Deprecated
	protected Color getExpotentialColor(double relativeLocation) {
		return new Color(null, expotentialRange(endColor.getRed(), startColor
				.getRed(), relativeLocation), expotentialRange(endColor
				.getBlue(), startColor.getBlue(), relativeLocation),
				expotentialRange(endColor.getGreen(), startColor.getGreen(),
						relativeLocation));
	}

	@Deprecated
	protected Color getLinearColor(double relativeLocation) {
		return new Color(null, linearRange(endColor.getRed(), startColor
				.getRed(), relativeLocation), linearRange(endColor.getBlue(),
				startColor.getBlue(), relativeLocation), linearRange(endColor
				.getGreen(), startColor.getGreen(), relativeLocation));
	}

	/*
	 * Calculates the appropriate colour for the given values time colouring
	 * using an exponential based algorithm
	 * 
	 * @author Colin Fagg and Linda Pettigrew
	 * 
	 * @date 23 April 2009
	 */
	private double calcRelativeTimeColor(long value) {
		if (value > endTime) {
			value = endTime;
		} else if (value < beginTime) {
			value = beginTime;
		}
		if (beginTime >= endTime) {
			System.err.println("error in time Begin>=end");
			return -1;
		}
		// value of x is the exponent which is used in the model
		return ((double) (value - beginTime))
				/ ((double) (endTime - beginTime));
	}

	private Color getEndColor() { // newer code
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		// System.out.println("Storage Colour End " + store.getInt("new_r") +
		// " "+ store.getInt("new_g") +" "+ store.getInt("new_b"));
		// return new Color (null, 200,200,200);
		// return new Color(null, store.getInt("new_r"),store.getInt("new_g"),
		// store.getInt("new_b"));
		return new Color(null, PreferenceConverter.getColor(store, "new_color"));
	}

	private Color getStartColor() { // older code
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		// System.out.println("Stoage Colour Start " + store.getInt("old_r") +
		// " "+ store.getInt("old_g") +" "+ store.getInt("old_b"));
		// return new Color(null, 100,100,100);
		return new Color(null, PreferenceConverter.getColor(store, "old_color"));
	}

	private long getEndTime() {// newest time
		// TODO Auto-generated method stub
		return 1;
	}

	private long getBeginTime() {// oldest time (furtherest in the past)
		// TODO Auto-generated method stub
		return 0;
	}

	// not needed at the moment.
	private class TimeAnnotationModelListener implements
			IAnnotationModelListenerExtension, IAnnotationModelListener {

		@Override
		public void modelChanged(AnnotationModelEvent event) {
			Annotation[] annos = event.getChangedAnnotations();
			annos = event.getAddedAnnotations();

		}

		@Override
		public void modelChanged(IAnnotationModel arg0) {

		}
	}

	/*
	 * Returns a long value for the information given by the annotation tag
	 * 
	 * @author Colin Fagg and Linda Pettigrew
	 * 
	 * @date 23 April 2009
	 */
	private long parseLongFromString(String tag, String text) {
		int endOfTag = text.indexOf(tag) + tag.length() + 1;
		String tempStr = text.substring(endOfTag, text.indexOf("\n", endOfTag));
		return Long.valueOf(tempStr).longValue();
	}

	public void setCodeHighlighting(int value) {

		coloringType = value;
		System.out.println("this is working!!");
		textWidget.redraw();

	}

	public void setColoring(boolean value) {
		colour = value;

		if (colour) {
			IPreferenceStore store = EditorsUI.getPreferenceStore();
			store
					.setValue(
							AbstractDecoratedTextEditorPreferenceConstants.EDITOR_CURRENT_LINE,
							false);
		} else {
			IPreferenceStore store = EditorsUI.getPreferenceStore();
			store
					.setValue(
							AbstractDecoratedTextEditorPreferenceConstants.EDITOR_CURRENT_LINE,
							true);
		}

		textWidget.redraw();
	}

	public CompilationUnitEditor getEditor() {

		return editor;
	}

}
