package metricsOverlay.handlers;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


import org.eclipse.core.filebuffers.FileBuffers;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

/**
 * This class keeps track of preference settings across different editor
 * parts.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
@SuppressWarnings("restriction")
public class DocumentTracker {
	
	private Set<MetricFile> mFiles;
	
	private static DocumentTracker dTracker;
	
	private DocumentTracker(){
		mFiles = new HashSet<MetricFile>();
	}
	
	public static DocumentTracker getInstance(){
		if(dTracker == null)
			dTracker = new DocumentTracker();
		return dTracker;
	}
	
	public void addMetricFile(MetricFile mFile){
		if(mFile != null){
		//We we already have a copy, clear it first.
		if(getFile(mFile.getPath()) != null)
			remove(mFile.getPath());
		//Add the new file in its place
		mFiles.add(mFile);
		}
	}
	
	public Set<MetricFile> getAllMetricFiles(){
		return mFiles;
	}
	
	public MetricFile getFile(IPath path){
		Iterator<MetricFile> it = mFiles.iterator();
		MetricFile found = null;
		while(it.hasNext()){
			MetricFile cur = it.next();
			if(cur.getPath().equals(path))
					found = cur;
		}
		return found;
	}
	
	public void remove(IPath path) {
		MetricFile mf = getFile(path);
		mFiles.remove(mf);
	}
	
	public int numFiles(){
		return mFiles.size();
	}
	
	/**
	 * A static method to get the workbench part from a document.
	 * 
	 * @param doc the document
	 * @return the workbench part corresponding to the document
	 */
	public static IWorkbenchPart getWorkbenchPart(IDocument doc, boolean restore){
		IWorkbenchPart finalPart = null;
		IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
		for(IWorkbenchWindow window: windows){
			for(IWorkbenchPage page: window.getPages()){
				for(IEditorReference ref: page.getEditorReferences()){
					IWorkbenchPart part = (IWorkbenchPart)ref.getEditor(restore);
					if(part instanceof JavaEditor){
						IDocument foundDoc = ((JavaEditor) part).getViewer().getDocument();
						if(doc.toString().equals(foundDoc.toString()))
							finalPart = part;
					}
				}
			}
		}
		return finalPart;
	}
	
	public static IWorkbenchPart getActiveJavaEditor() {
		IWorkbenchPart part = (IWorkbenchPart)PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if(part instanceof JavaEditor)
			return part;
		else 
			return null;
	}
	
	public static IDocument getDocumentFromPart(IWorkbenchPart part){
		return ((JavaEditor) part).getViewer().getDocument();
	}
	
	public static IPath getPathFromDocument(IDocument doc){
		return FileBuffers.getTextFileBufferManager().getTextFileBuffer(doc).getLocation();
	}
	
	public static IPath convertRelativePathToAbsolute(IPath path){
		IWorkspace ws = ResourcesPlugin.getWorkspace();
		IFile f = null;
		for(IProject p : ws.getRoot().getProjects()){
			f = p.getFile(path.removeFirstSegments(1));
			if(f != null)
				break;
		}
		String file = f.getLocationURI().toString();
		IPath ped = Path.fromOSString(file);
		return ped;
	}

	public boolean contains(MetricFile mf) {
		return mFiles.contains(mf);
	}

	
}
