package metricsOverlay.handlers;

import metricsOverlay.model.MetricDocument;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IWorkbenchPart;

import editorListener.handlers.EditorState;
import editorListener.handlers.UpdatedEditor;

/**
 * This class is a high level data container containing
 * the model and the JavaEditor parts that this model is
 * designed to work over.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class MetricFile implements Comparable<MetricFile> {
	
	private MetricDocument mDoc;
	private UpdatedEditor edit;
	private boolean inFocus;
	
	public MetricFile(IDocument doc, IWorkbenchPart part, MetricDocument mDoc,
			IPath path, boolean inFocus) {
		this.mDoc = mDoc;
		edit = new UpdatedEditor(doc, part, EditorState.DocumentModified);
		this.inFocus = inFocus;
	}
	
	public MetricFile(MetricDocument mDoc, UpdatedEditor uEdit) {
		this.mDoc = mDoc;
		this.edit = uEdit;
		this.inFocus = true;
	}
	
	//Getters/Setters
	
	public IDocument getDocument() {
		return edit.getDocument();
	}
	public IWorkbenchPart getWorkbenchPart() {
		return edit.getWorkbenchPart();
	}
	public MetricDocument getMetricDocument() {
		return mDoc;
	}
	public IPath getPath() {
		return edit.getPath();
	}
	public boolean isInFocus() {
		return inFocus;
	}

	@Override
	public int compareTo(MetricFile o) {
		if(mDoc.equals(o.mDoc) && edit.equals(o.edit))
			return 1;
		else if(mDoc.equals(o.mDoc))
			return 0;
		else
			return -1;
	}

	public EditorState getState() {
		return edit.getTheState();
	}

}
