package metricsOverlay.handlers;

import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.metrics.DerivedMetric;
import metricsOverlay.model.metrics.Metric;

/**
 * This class deals with cases where the model has been changed internally and
 * needs to be rationalised with new input.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class ModelRationaliser {

	public MetricFile rationalise(MetricFile oldFile, MetricFile newFile) {
		// add all augmentations to mf
		for (Augmentation aug : oldFile.getMetricDocument()
				.getAllAugmentations()) {
			String forRemoval = null;
			for (Augmentation newAug : newFile.getMetricDocument()
					.getAllAugmentations()) {
				if (aug.getName().equals(newAug.getName())) {
					forRemoval = newAug.getName();
					break;
				}
			}
			if (forRemoval != null)
				newFile.getMetricDocument().removeAugmentation(forRemoval);
			newFile.getMetricDocument().addAugmentation(aug);
		}
		
		// add all derived metrics to mf
		for (Metric met : oldFile.getMetricDocument().getAllMetrics()) {
			if (met instanceof DerivedMetric) {
				String forRemoval = null;
				for (Metric newMet : newFile.getMetricDocument()
						.getAllMetrics()) {
					if (met.getName().equals(newMet.getName())) {
						forRemoval = newMet.getName();
						break;
					}
				}
				if (forRemoval != null)
					newFile.getMetricDocument().removeMetric(forRemoval);
				newFile.getMetricDocument().addMetric(met);
			}
		}
		
		// add all mappings to mf
		for (Mapping map : oldFile.getMetricDocument().getAllMappings()) {
			String forRemoval = null;
			for (Mapping newMap : newFile.getMetricDocument().getAllMappings()) {
				if (map.getName().equals(newMap.getName())) {
					forRemoval = newMap.getName();
					break;
				}
			}
			if (forRemoval != null)
				newFile.getMetricDocument().removeMapping(forRemoval);
			newFile.getMetricDocument().addMapping(map);
		}
		
		//keep mapping connections up-to-date.
		for(Mapping map : newFile.getMetricDocument().getAllMappings()){
			Metric m = map.getMetric();
			if(m != null){
				Metric newM = newFile.getMetricDocument().getMetric(m.getName());
				map.setMetric(newM);
			}
			Augmentation a = map.getAugmentation();
			if(a != null) {
				Augmentation newA = newFile.getMetricDocument().
					getAugmentation(a.getName());
				map.setAugmentation(newA);
			}
			
		}
		

		
		return newFile;
	}
}
