package metricsOverlay.model;

/**
 * This interface allows items to be flagged as editable and have the potential
 * to change their editable status.
 * 
 * Please note this is only a flag and it is up to users of the class to set
 * this if desired.
 * 
 * @author Matthew Harward
 * @version 28/8/2009
 */
public interface IEditable {

	/**
	 * @return A boolean value that determines if you are allowed to edit this
	 *         class.
	 */
	public boolean canEdit();

	/**
	 * @return A value to determine if the editable status may be changed.
	 */
	public boolean canChangeEditableStatus();

	/**
	 * If allowed, this sets the new editable status.
	 * 
	 * @param editable
	 *            the new status.
	 */
	public void setEditable(boolean editable);
}
