package metricsOverlay.model;

/**
 * This class represents a single metric section.
 * 
 * @author Matthew Harward
 * @version 14/7/2009
 */
public class MetricSection implements Comparable<MetricSection> {

	// Fields
	private int startLine;
	private int endLine;
	private int startCol;
	private int endCol;
	private String value;

	// Optional Fields
	private String actualText;
	private String annotation;

	// Currently unused
	@SuppressWarnings("unused")
	private SectionType stype;

	/**
	 * This is the main constructor for a metric section.
	 * 
	 * @param startLine
	 *            the actual line this metric section starts on.
	 * @param startCol
	 *            the actual column this metric section starts on.
	 * @param endLine
	 *            the actual line this metric section ends on, must be equal or
	 *            greater than the start line.
	 * @param endCol
	 *            the actual column this metric section ends on. If the start
	 *            line is equal to the end line, this value must be larger than
	 *            or equal to the start col.
	 * @param value
	 *            the value, in a String format, that this metric section has.
	 * @throws Exception
	 *             if any value is not valid
	 */
	public MetricSection(int startLine, int startCol, int endLine, int endCol,
			String value) throws Exception {
		this.startLine = startLine;
		this.startCol = startCol;
		this.endLine = endLine;
		this.endCol = endCol;
		this.value = value;
		if (!checkValidity())
			throw new Exception();
		else
			assignType();
	}

	/**
	 * Full constructor
	 */
	public MetricSection(int startLine, int endLine, int startCol, int endCol,
			String value, String actualText, String annotation) throws Exception {
		this(startLine,startCol, endLine ,endCol,value);
		this.actualText = actualText;
		this.annotation = annotation;
	}

	// This method is currently strictly unnecessary..
	private void assignType() {
		if (startLine < endLine)
			stype = SectionType.SECTION;
		if (startLine == endLine)
			stype = SectionType.LINE;
		// Constant line length, currently, proper check should be added later
		int LINELENGTH = 60;
		if (startLine == endLine && (endCol - startLine) < LINELENGTH)
			stype = SectionType.SECTION;
	}

	// This method determines if the start and end points of the
	// metric section are valid.
	private boolean checkValidity() {
		boolean test = false;
		if (startLine == endLine && startCol <= endCol)
			test = true;
		else if (startLine < endLine)
			test = true;
		return test;
	}

	// Getters & Setters
	public int getEndLine() {
		return endLine;
	}

	public void setEndLine(int endLine) {
		this.endLine = endLine;
	}

	public int getStartCol() {
		return startCol;
	}

	public void setStartCol(int startCol) {
		this.startCol = startCol;
	}

	public int getEndCol() {
		return endCol;
	}

	public void setEndCol(int endCol) {
		this.endCol = endCol;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public int getStartLine() {
		return startLine;
	}

	public void setStartLine(int startLine) {
		this.startLine = startLine;
	}

	public String getActualText() {
		return actualText;
	}

	public void setActualText(String actualText) {
		this.actualText = actualText;
	}

	public boolean hasActualText() {
		if (actualText != null)
			return true;
		else
			return false;
	}
	
	public boolean hasAnnotation() {
		if (annotation != null)
			return true;
		else
			return false;
	}

	public void setAnnotation(String annotation) {
		this.annotation = annotation;
	}

	public String getAnnotation() {
		return annotation;
	}

	/**
	 * Overridden toString method
	 */
	public String toString() {
		return "MetricSection: [" + value + "] " + startLine + "c" + startCol
				+ " - " + endLine + "c" + endCol;
	}

	@Override
	public int compareTo(MetricSection o) {
		if(value == o.value && startLine == o.startLine && startCol == o.startCol
				&& endLine == o.endLine && endCol == o.endCol)
			return 1;
		else return -1;
	}
}
