package metricsOverlay.model.augmentation;

import java.util.ArrayList;
import java.util.List;

import metricsOverlay.colorManagers.ContinuousListColorManager;
import metricsOverlay.colorManagers.DiscreteListColorManager;
import metricsOverlay.colorManagers.IColorManager;
import metricsOverlay.colorManagers.OneColorManager;
import metricsOverlay.colorManagers.TwoColorManager;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.IEditable;
import metricsOverlay.model.INameable;
import metricsOverlay.model.ITyped;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.swt.graphics.Color;

/**
 * This class provides a representation of a display.
 * 
 * @author Matthew Harward
 * @version 8/7/2009
 */
public abstract class Augmentation implements Comparable<Augmentation>, 
	ITyped, INameable, IEditable {
	
	//Private fields
	protected String augName;
	//protected AugmentationStrategy strategy;
	protected boolean discrete;
	protected boolean annotations;
	protected int importance;
	protected List<Color> colors;
	protected IColorManager cManager;
	private boolean editable;
	
	/*
	private AugmentationType type;
	
	type
	type -> name
	type -> area it can work over
	type -> how to instantiate an object of that */
	
	public Augmentation() {
		this("Default");
	}
	
	/**
	 * Main constructor
	 * 
	 * @param augName a name defining an implemented type of display
	 * strategy. Existing strategies can be found in AugmentationName.
	 */
	public Augmentation(String augName) {
		this.augName = augName;
		discrete = false;
		annotations = true;
		colors = new ArrayList<Color>();
		importance = 50;
		editable = true;
		//strategy = AugmentationStrategyFactory.instance(dType);
	}
	
	/**
	 * Gets the display strategy used by this display.
	 * 
	 * @return the strategy used by this display.
	 */
	//public AugmentationStrategy getStrategy(){
	//	return strategy;
	//}

	/**
	 * The name of this display
	 * 
	 * @return the name
	 */
	public String getName()
	{
		return augName;
	}
	
	public String getType(){
		return "Abstract Augmentation";
	}
	
	/**
	 * This to string method provides a textual representation of the object.
	 */
	public String toString()
	{
		return "Augmentation: " + getName();
	}

	public abstract void disable();// {
		//System.out.println("Disabling : " + toString());
		//strategy.dispose();
	//}

	@Override
	public int compareTo(Augmentation o) {
		if(augName.equals(o.augName))
			return 1;
		else return -1;
	}
	
	public void addColor(Color color) {
		colors.add(color);
	}

	public void setName(String name) {
		augName = name;
	}

	public int getImportance() {
		return importance;
	}
	
	/**
	 * This method updates the display strategy, given an 
	 * update in the eclipse java editor.
	 * 
	 * @param mf the updated metric file.
	 * @param mm the name of the mapping this display applies to.
	 */
	public abstract void update(MetricFile mf, Mapping mm);// {
		//strategy.update(mf, mm);
	//}

	public boolean isAnnotated() {
		return annotations;
	}

	public boolean isDiscrete() {
		return discrete;
	}

	public void setImportance(int importance) {
		int im = importance;
		if(im > 100)
			im = 100;
		if(im < 0)
			im = 0;
		this.importance = im;
	}

	public void setDiscrete(boolean discrete) {
		this.discrete = discrete;
		
	}

	public void setAnnotations(boolean annotations) {
		this.annotations = annotations;
	}

	public double getAugMax() {
		return getCManager().getRange();
	}

	public double getAugMin() {
		return 0;
	}
	
	public IColorManager getCManager(){
		if(cManager == null)
			cManager = makeNewCManager();
		return cManager;
	}

	private IColorManager makeNewCManager() {
		IColorManager man = null;
		if(colors.size() == 1){
			man = new OneColorManager(colors.get(0));
		} else if(discrete) {
			man = new DiscreteListColorManager(colors);
		} else if (colors.size() == 2) {
			man = new TwoColorManager(colors.get(0), colors.get(1));
		} else {
			man = new ContinuousListColorManager(colors);
		}
		return man;
	}

	public List<Color> getColors() {
		return colors;
	}

	public void setColors(List<Color> colors) {
		this.colors = colors;	
	}
	
	@Override
	public boolean canChangeEditableStatus() {
		return true;
	}

	@Override
	public boolean canEdit() {
		return editable;
	}

	@Override
	public void setEditable(boolean editable) {
		this.editable = editable;
	}
}
