package metricsOverlay.model.augmentation;

import java.util.Set;

import metricsOverlay.augmentationStrategies.MarginChipStrategy;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.augmentation.other.MarginController;
import metricsOverlay.model.augmentation.other.ShapeDirection;
import metricsOverlay.model.augmentation.other.ShapeManager;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.mappings.Mapping;


public class MarginAugmentation extends Augmentation {

	private boolean isleft;
	private MarginChipStrategy mStrat;
	private ShapeType sType;
	private ShapeDirection sDir;
	
	public MarginAugmentation() {
		super();
		isleft = true;
		sType = ShapeType.roundedsquare;
		sDir = ShapeDirection.left;
	}
	
	public MarginAugmentation(String augName) {
		super(augName);
		isleft = true;
		sType = ShapeType.roundedsquare;
		sDir = ShapeDirection.left;
	}

	public void setShape(ShapeType sType, ShapeDirection sDir){
		this.sType = sType;
		this.sDir = sDir;
	}

	public boolean isLeft() {
		return isleft;
	}
	
	public void setIsLeft(boolean left) {
		isleft = left;
	}
	
	public int getMarginPosition(){
		return MarginController.getCurInstance().getAugmentationPosition(this);
	}

	public String getType() {
		return "Margin Augmentation";
	}

	@Override
	public void disable() {
		if(mStrat != null)
			mStrat.dispose();
		mStrat = null;
	}

	@Override
	public void update(MetricFile mf, Mapping mm) {
		if(mStrat == null)
			mStrat = new MarginChipStrategy();
		mStrat.update(mf, mm);
		
	}

	public int totChipsOnSide() {
		return MarginController.getCurInstance().numChipsOnSide(this);
	}

	public ShapeType getShapeType() {
		return sType;
	}

	public Set<int[]> getShape(int lineHeight) {
		return ShapeManager.getShape(lineHeight, sType, sDir);
	}

	public ShapeDirection getShapeDirection() {
		return sDir;
	}

}