package metricsOverlay.model.augmentation.other;

import java.util.HashSet;
import java.util.Set;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;

public class AugmentationManager {

	private MetricFile curMF;
	private Set<Mapping> activeDisplays;
	private MarginController mControl;
	
	public AugmentationManager() {
		curMF = null;
		activeDisplays = new HashSet<Mapping>();
		mControl = MarginController.getCurInstance();
	}
	
	public void updateDisplay(MetricFile mf) {
		if(mf != null){
		curMF = mf;
		Set<Mapping> pastDisplays = new HashSet<Mapping>(activeDisplays);
		clearAllDisplays();
		for(Mapping mm : curMF.getMetricDocument().getAllMappings()) {
			if(mm.getMappingState().equals(MappingState.ActiveMapping)){
				activeDisplays.add(mm);
			} else if(pastDisplays.contains(mm)) {
				//If inactive and in past displays, then dispose
				mm.getAugmentation().disable();
			}
		}
		mControl.generateNewMargins(activeDisplays, true);
		for(Mapping mm : activeDisplays) {
			mm.getAugmentation().update(mf, mm);
		}
		}
	}
	
	public void clearAllDisplays() {
		for(Mapping mm: activeDisplays){
			mm.getAugmentation().disable();
		}
		activeDisplays.clear();
	}
}
