package metricsOverlay.model.augmentation.other;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;

public class MarginController {
	
	private static MarginController mController;
	//position from outside, from 0
	private Map<MarginAugmentation,Integer> leftMarginPositions;
	private Map<MarginAugmentation,Integer> rightMarginPositions;
	
	private MarginController(){
		leftMarginPositions = new HashMap<MarginAugmentation,Integer>();
		rightMarginPositions = new HashMap<MarginAugmentation,Integer>();
	}
	
	public void generateNewMargins(Set<Mapping> mappings, boolean favourInner){
		leftMarginPositions.clear();
		rightMarginPositions.clear();
		List<Mapping> mapListLeft = new ArrayList<Mapping>();
		List<Mapping> mapListRight = new ArrayList<Mapping>();
		//Get all relevant mappings
		for(Mapping map : mappings){
			if(map.getMappingState().equals(MappingState.ActiveMapping)){
				if(map.getAugmentation() instanceof MarginAugmentation){
					if(((MarginAugmentation)map.getAugmentation()).isLeft()){
						mapListLeft.add(map);
					} else {
						mapListRight.add(map);
					}
					
				}
			}
		}
		//Allocate positions
		leftMarginPositions.putAll(extractPositions(favourInner, mapListLeft));
		rightMarginPositions.putAll(extractPositions(favourInner, mapListRight));
	}

	/**
	 * @param favourInner
	 * @param mapList
	 */
	private Map<MarginAugmentation,Integer> extractPositions(boolean favourInner, List<Mapping> mapList) {
		Map<MarginAugmentation,Integer> mPos1 = new HashMap<MarginAugmentation,Integer>();
		for(Mapping map: mapList){
			int importance = map.getAugmentation().getImportance();
			if(!favourInner)
			{
				importance = 100 - importance;
			}
			while(mPos1.containsValue(importance)){
				importance--;
			}
			MarginAugmentation mAug = (MarginAugmentation)map.getAugmentation();
			mPos1.put(mAug, importance);
		}
		List<Integer> valueList = new ArrayList<Integer>(mPos1.values());
		Collections.sort(valueList);
		for(MarginAugmentation mAug : mPos1.keySet()){
			int index = valueList.indexOf(mPos1.get(mAug));
			mPos1.put(mAug, index);
		}
		return mPos1;
	}
	
	public int getAugmentationPosition(MarginAugmentation mAug){
		if(leftMarginPositions.containsKey(mAug)){
			return leftMarginPositions.get(mAug);
		}else if(rightMarginPositions.containsKey(mAug)){
				return rightMarginPositions.get(mAug);
		} else return -1;
	}

	public static MarginController getCurInstance() {
		if(mController == null)
			mController = new MarginController();
		return mController;
	}

	public int numChipsOnSide(MarginAugmentation mAug) {
		if(leftMarginPositions.containsKey(mAug)){
			return leftMarginPositions.size();
		}else if(rightMarginPositions.containsKey(mAug)){
				return rightMarginPositions.size();
		} else return -1;
	}
	
}
