package metricsOverlay.model.mappings;

import metricsOverlay.model.augmentation.Augmentation;

/**
 * This class provides the functionality to interpolate between different metric
 * functionality. Currently this class parrots back the existing
 * <code>Augmentation</code> values.
 * 
 * This class should be subclassed if additional interpolation facilities are
 * required.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class AugmentationInterpolator {

	protected Augmentation dStrat;

	/**
	 * Main constructor.
	 * 
	 * @param dStrat
	 *            the augmentation to be used.
	 */
	public AugmentationInterpolator(Augmentation dStrat) {
		this.dStrat = dStrat;
	}

	/**
	 * @return the max value for the current augmentation
	 */
	public double getMax() {
		return dStrat.getAugMax();
	}

	/**
	 * @return the min value for the current augmentation
	 */
	public double getMin() {
		return dStrat.getAugMin();
	}

	/**
	 * @param value
	 *            the value produced by the mapping interpolator.
	 * @return the value to be used by the augmentation.
	 */
	public int interpolateValue(int value) {
		return value;
	}

}
