package metricsOverlay.model.mappings;

import metricsOverlay.model.ITyped;

/**
 * This class represents an abstract interpolation strategy for interpolating
 * between a metric value and a display value.
 * 
 * @author Matthew Harward
 * @version 10/8/09
 */
public abstract class InterpolationStrategy implements ITyped {

	protected double dMax;
	protected double dMin;
	protected double mMax;
	protected double mMin;
	
	/**
	 * All non abstract subclasses must have a blank constructor.
	 */
	public InterpolationStrategy() {
	}
	
	public InterpolationStrategy(double dMax, double dMin, double mMax, double mMin) {
		this.dMax = dMax;
		this.dMin = dMin;
		this.mMax = mMax;
		this.mMin = mMin;
	}
	
	public abstract int getInterpolatedValue(String value);
	
	public abstract String getType();

	public void setMinMax(double dMax, double dMin, double mMax, double mMin) {
		this.dMax = dMax;
		this.dMin = dMin;
		this.mMax = mMax;
		this.mMin = mMin;
	}

}
