package metricsOverlay.model.mappings;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import metricsOverlay.augmentationStrategies.AugmentationStrategy;

/**
 * A class to manage the properties of a mapping.
 * 
 * @author Matthew Harward
 * @version 28/6/2009
 */
@Deprecated
public class MappingProperties {

	// The map
	private Map<String, String> propMap;

	// Defined Values
	protected static final String DESCRIPTION = "Description";
	protected static final String MAPPINGTYPE = "MappingType";
	protected static final String EXPONENTIALVALUE = "ExponentialValue";
	protected static final String LOGVALUE = "LogValue";

	// Defined Mapping Types
	protected static final String LINEAR = "Linear";
	protected static final String METRICEXPONENTIAL = "MetricExponential";
	protected static final String DISPLAYEXPONENTIAL = "DisplayExponential";
	protected static final String METRICLOG = "MetricLog";
	protected static final String DISPLAYLOG = "DisplayLog";

	// Defaults
	protected static final String DEFAULTEXPONENTIAL = "2.0";
	protected static final String DEFAULTLOG = "10.0";
	protected static final String TRUE = "true";
	protected static final String ONE = "1";
	protected static final String ZERO = "0";
	
	// Defined Augmentation Constants
	public static final String COLOR = "color"; //Multiple allowed 'Color1', 'Color2'
	public static final String ONLEFT = "On Left";
	public static final String NUMCHIPS = "Number Of Chips"; //On this side
	public static final String CHIPPOSITION = "Chip Position"; //On this side
	public static final String SHAPE = "Shape";

	//Underline options
	//TODO validate
	public static final String UNDERLINETYPE = "Underline Type";
	public static final String ULINE_LINK = "link";
	public static final String ULINE_SQUIGGLE = "squiggle";
	public static final String ULINE_SINGLE = "single";
	public static final String ULINE_DOUBLE = "double";

	/**
	 * Default constructor.
	 */
	protected MappingProperties() {
		propMap = new HashMap<String, String>();
	}

	/**
	 * Main constructor.
	 * 
	 * @param properties
	 *            a map containing the properties of the map.
	 */
	protected MappingProperties(Map<String, String> properties) {
		this.propMap = properties;
		if(propMap == null){
			propMap = new HashMap<String,String>();
		}
		validateMap();
	}

	/**
	 * This method validates the properties map.
	 */
	private void validateMap() {
		// Updated value map
		Map<String, String> updatedProps = new HashMap<String, String>();
		// Iterate through properties
		Iterator<String> it = propMap.keySet().iterator();
		while (it.hasNext()) {
			String s = it.next();
			String value = propMap.get(s);
			if (s.equalsIgnoreCase(DESCRIPTION)) {
				updatedProps.put(DESCRIPTION, value);
			} else if (s.equalsIgnoreCase(MAPPINGTYPE)) {
				if (value.equalsIgnoreCase(LINEAR))
					value = LINEAR;
				else if (value.equalsIgnoreCase(METRICEXPONENTIAL))
					value = METRICEXPONENTIAL;
				else if (value.equalsIgnoreCase(DISPLAYEXPONENTIAL))
					value = DISPLAYEXPONENTIAL;
				else if (value.equalsIgnoreCase(METRICLOG))
					value = METRICLOG;
				else if (value.equalsIgnoreCase(DISPLAYLOG))
					value = DISPLAYLOG;
				else
					value = LINEAR;
				updatedProps.put(MAPPINGTYPE, value);
			} else if (s.equalsIgnoreCase(EXPONENTIALVALUE)) {
				try {
					Double.parseDouble(value);
				} catch (NumberFormatException e) {
					value = DEFAULTEXPONENTIAL;
				}
				updatedProps.put(EXPONENTIALVALUE, value);
			} else if (s.equalsIgnoreCase(LOGVALUE)) {
				try {
					Double.parseDouble(value);
				} catch (NumberFormatException e) {
					value = DEFAULTLOG;
				}
				updatedProps.put(LOGVALUE, value);
			} else if(s.toLowerCase().startsWith(COLOR)) {
				try {
					int cur = Integer.parseInt(s.substring(COLOR.length()));
					if(AugmentationStrategy.parseColor(value) != null)
						updatedProps.put(COLOR+cur, value);
				} catch (NumberFormatException e){}
			} else if(s.equalsIgnoreCase(ONLEFT)) {
				try {
					Boolean.parseBoolean(value);
				} catch (NumberFormatException e) {
					value = TRUE;
				}
				updatedProps.put(ONLEFT, value);
			} else if (s.equalsIgnoreCase(NUMCHIPS)) {
				try {
					Integer.parseInt(value);
				} catch (NumberFormatException e) {
					value = ONE;
				}
				updatedProps.put(NUMCHIPS, value);
			} else if (s.equalsIgnoreCase(CHIPPOSITION)) {
				try {
					Integer.parseInt(value);
				} catch (NumberFormatException e) {
					value = ZERO;
				}
				updatedProps.put(CHIPPOSITION,value);
			} else if (s.equalsIgnoreCase(SHAPE)) {
				// No checking currently, defaults to rounded rectangle
				updatedProps.put(SHAPE, value);
			} else if (s.equalsIgnoreCase(UNDERLINETYPE)) {
				//No checking currently, defaults to squiggle
				updatedProps.put(UNDERLINETYPE, value);
			} else {	
				// If the property is not valid, remove property.
			}
			it.remove();
		}
		propMap.putAll(updatedProps);
	}

	/**
	 * Gets a property by name.
	 * 
	 * @param name
	 *            the name of the property
	 * @return the value of the property
	 */
	public String getProperty(String name) {
		return propMap.get(name);
	}

	/**
	 * Adds a property, given a name and value. This name and value need to be
	 * valid.
	 * 
	 * @param name
	 *            the name of the property to add.
	 * @param value
	 *            the corresponding value.
	 */
	public void addProperty(String name, String value) {
		propMap.put(name, value);
		validateMap();
	}

	/**
	 * Returns a modifiable map for the mapping.
	 * 
	 * @return the map.
	 */
	public Map<String, String> getAllProperties() {
		return propMap;
	}
	
	/**
	 * Returns a string representation of the properties
	 */
	public String toString(){
		return "Properties: " + propMap.toString();
	}
}
