package metricsOverlay.model.metrics;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import metricsOverlay.model.MetricSection;

/**
 * This class represents and holds actual metrics data.
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public class ActualMetric extends Metric {

	//The actual metric parts
	Set<MetricSection> mSections;
	
	/**
	 * A reduced constructor. Sets up a default IntegerRange.
	 * 
	 * @param name the unique name of the metric.
	 */
	public ActualMetric(String name)
	{
		this(name,new IntegerRange(1,100));
	}
	
	/**
	 * Main constructor.
	 * 
	 * @param mname the unique name of the metric.
	 * @param mrange the range of the metric.
	 */
	public ActualMetric(String mname, Range mrange)
	{
		name = mname;
		range = mrange;
		mSections = new HashSet<MetricSection>();
	}
	
	/**
	 * @param mSections
	 *            Replace all of the current metric sections with this set
	 */
	public void setMetricSections(Set<MetricSection> mSections) {
		this.mSections = mSections;
	}

	/**
	 * @param ms
	 *            The metric section to be added.
	 */
	public void addMetricSection(MetricSection ms) {
		mSections.add(ms);
	}

	/**
	 * @param ms
	 *            The metric section to be removed
	 */
	public void removeMetricSection(MetricSection ms) {
		mSections.remove(ms);
	}

	/**
	 * @return An unmodifiable set of MetricSections
	 */
	public Set<MetricSection> getMetricSections() {
		return Collections.unmodifiableSet(mSections);
	}
	
	/**
	 * @return The number of MetricSections associated with this mapping
	 */
	public int numSections() {
		return mSections.size();
	}
	
	public String toString()
	{
		return name;
	}

	@Override
	public boolean canChangeEditableStatus() {
		return false;
	}

	@Override
	public boolean canEdit() {
		return false;
	}

	@Override
	public void setEditable(boolean editable) {
		//Null Opp Override
		//Tut tut
	}
}
