package metricsOverlay.model.metrics;

import java.util.HashMap;
import java.util.Map;

/**
 * This class provides the ability to use complex
 * range data that is composed of different integer ranges.
 * 
 * Potentially composite real ranges may also be avaliable.
 * 
 * @author Matthew Harward
 * @version 1/9/2009
 */
public class CompositeIntegerRange extends MappedRange {
	
	private Map<IntegerRange,String> cRange;
	private boolean initialised;
	
	public CompositeIntegerRange() {
		cRange = new HashMap<IntegerRange,String>();
		initialised = false;
	}
	
	public CompositeIntegerRange(Map<IntegerRange, String> ranges) {
		cRange = ranges;
		initialised = true;
	}
	
	@Override
	public boolean isValid() {
		if(!initialised)
			return false;
		boolean validRange = true;
		//Quick initial check to make sure the inside range is valid.
		for(IntegerRange ir : cRange.keySet()){
			if(!ir.isValid()){
				return false;
			}	
		}
		int numOccurances = 0;
		for(int i = Integer.parseInt(getMin()); i <= Integer.parseInt(getMax()); i++){
			for(IntegerRange ir : cRange.keySet()){
				if(ir.isInRange(String.valueOf(i)))
					numOccurances++;
			}
			if(numOccurances > 1){
				validRange = false;
				break;
			}
			numOccurances = 0;
		}
		return validRange;
	}

	@Override
	public String getMax() {
		int max = Integer.MIN_VALUE;
		for(IntegerRange a : cRange.keySet()){
			int cur = 0;
			if((cur = Integer.parseInt(a.getMax())) > max)
				max = cur;
		}
		return String.valueOf(max);
	}

	@Override
	public String getMin() {
		int min = Integer.MAX_VALUE;
		for(IntegerRange a : cRange.keySet()){
			int cur = 0;
			if((cur = Integer.parseInt(a.getMin())) < min)
				min = cur;
		}
		return String.valueOf(min);
	}

	public String getMaxName(){
		String maxName = null;
		String maxVal = getMax();
		for(IntegerRange a : cRange.keySet()){
			if(a.getMax().equals(maxVal)){
				maxName = cRange.get(a);
				break;
			}
		}
		return maxName;
	}

	public String getMinName(){
		String minName = null;
		String minVal = getMin();
		for(IntegerRange a : cRange.keySet()){
			if(a.getMin().equals(minVal)){
				minName = cRange.get(a);
				break;
			}
		}
		return minName;
	}

	@Override
	public String getType() {
		return "Composite Integer Range";
	}

	@Override
	public boolean isInRange(String value) {
		boolean inRange = false;
		for(IntegerRange a : cRange.keySet()){
			if(a.isInRange(value)){
				inRange = true;
				break;
			}
		}
		return inRange;
	}
	
	public boolean isInRangeName(String value) {
		if (cRange.containsValue(value))
			return true;
		else
			return false;
	}
	
	public Map<IntegerRange,String> getRangeMap() {
		return cRange;
	}

	@Override
	public int getNumItems() {
		return cRange.size();
	}
}
