package metricsOverlay.model.metrics;

import java.util.HashMap;
import java.util.Map;

/**
 * This class provides the ability to use complex
 * range data that is composed of different real ranges.
 * 
 * Potentially composite real ranges may also be available.
 * 
 * @author Matthew Harward
 * @version 1/9/2009
 */
public class CompositeRealRange extends MappedRange {
	
	private Map<RealRange,String> cRange;
	private boolean initialised;
	
	public CompositeRealRange() {
		cRange = new HashMap<RealRange,String>();
		initialised = false;
	}
	
	public CompositeRealRange(Map<RealRange, String> ranges) {
		cRange = ranges;
		initialised = true;
	}
	
	@Override
	public boolean isValid() {
		if(!initialised)
			return false;
		boolean validRange = true;
		int count = 0;
		//As no modification takes place, no ConcurrentModificationExceptions
		//should occur.
		for(RealRange rr : cRange.keySet()){
			if(!rr.isValid()){
				validRange = false;
				break;
			}
			for(RealRange rr2 : cRange.keySet()){
				//If they fall inside each other
				if(rr.isInRange(rr2.getMax()) || rr.isInRange(rr2.getMin()) ||
						rr2.isInRange(rr.getMin()) || rr2.isInRange(rr.getMax())){
					count++;
				}
			}
			if(count > 1)
				validRange = false;
			count = 0;
		}
		return validRange;
	}

	@Override
	public String getMax() {
		double max = Double.MIN_VALUE;
		for(RealRange a : cRange.keySet()){
			double cur = 0;
			if((cur = Double.parseDouble(a.getMax())) > max)
				max = cur;
		}
		return String.valueOf(max);
	}

	@Override
	public String getMin() {
		double min = Double.MAX_VALUE;
		for(RealRange a : cRange.keySet()){
			double cur = 0;
			if((cur = Double.parseDouble(a.getMin())) < min)
				min = cur;
		}
		return String.valueOf(min);
	}

	public String getMaxName(){
		String maxName = null;
		String maxVal = getMax();
		for(RealRange a : cRange.keySet()){
			if(a.getMax().equals(maxVal)){
				maxName = cRange.get(a);
				break;
			}
		}
		return maxName;
	}

	public String getMinName(){
		String minName = null;
		String minVal = getMin();
		for(RealRange a : cRange.keySet()){
			if(a.getMin().equals(minVal)){
				minName = cRange.get(a);
				break;
			}
		}
		return minName;
	}

	@Override
	public String getType() {
		return "Composite Real Range";
	}

	@Override
	public boolean isInRange(String value) {
		boolean inRange = false;
		for(RealRange a : cRange.keySet()){
			if(a.isInRange(value)){
				inRange = true;
				break;
			}
		}
		return inRange;
	}
	
	public boolean isInRangeName(String value) {
		if (cRange.containsValue(value))
			return true;
		else
			return false;
	}
	
	public Map<RealRange,String> getRangeMap() {
		return cRange;
	}

	@Override
	public int getNumItems() {
		return cRange.size();
	}
}
