package metricsOverlay.model.metrics;

import java.util.Set;

import metricsOverlay.model.MetricSection;

/**
 * This class represents a derived form of metric to allow displays to use the
 * same metric sections.
 * 
 * Note: It has been decided to only allow composition from ActualMetric
 * objects. This is because the process of wrapping adds no
 * additional functionality.
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public class DerivedMetric extends Metric {

	//The metric this metric is derived from
	private ActualMetric met;
	private boolean editable;

	/**
	 * Reduced constructor. Sets up a default integer range.
	 * 
	 * @param name the unique name of this metric.
	 * @param met the metric object that this one is derived from.
	 */
	public DerivedMetric(String name, ActualMetric met) {
		this(name, new IntegerRange(1, 100), met);
	}

	/**
	 * Main constructor.
	 * 
	 * @param name the unique name of the metric
	 * @param range the range to be used
	 * @param met the metric from whence this metric's data is derived.
	 */
	public DerivedMetric(String name, Range range, ActualMetric met) {
		this.name = name;
		this.range = range;
		this.met = met;
		editable = true;
	}

	/**
	 * @return An unmodifiable set of MetricSections
	 */
	public Set<MetricSection> getMetricSections() {
		return met.getMetricSections();
	}

	/**
	 * @return The number of MetricSections associated with this mapping
	 */
	public int numSections() {
		return met.numSections();
	}

	/**
	 * @return The name of the metric this is derived from.
	 */
	public String derivedFrom() {
		return met.getName();
	}

	@Override
	public boolean canChangeEditableStatus() {
		return true;
	}

	@Override
	public boolean canEdit() {
		return editable;
	}

	@Override
	public void setEditable(boolean editable) {
		this.editable = editable;
	}

	public void setUnderlyingMetric(ActualMetric actMet) {
		met = actMet;
	}

}
