package metricsOverlay.model.metrics;

/**
 * An ordinal range using integer numbers.
 * 
 * Note: This method does not check to make sure the max >= min
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public class IntegerRange extends Range {

	//Max and min fields
	private int min;
	private int max;
	private boolean initialised;
	
	/**
	 * Parameterless constructor. This should not be 
	 * called.
	 */
	public IntegerRange() {
		initialised = false;
	}
	
	/**
	 * Main constructor.
	 * @param min the minimum value.
	 * @param max the maximum value.
	 */
	public IntegerRange(int min, int max) {
		this.min = min;
		this.max = max;
		initialised = true;
	}

	@Override
	public String getMax() {
		return Integer.toString(max);
	}

	@Override
	public String getMin() {
		return Integer.toString(min);
	}

	@Override
	public boolean isInRange(String value) {
		Integer testVal = Integer.valueOf(value);
		boolean result = false;
		if(testVal == null)
			result = false;
		else if (testVal >= min && testVal <= max)
			result = true;
		return result;
	}
	
	@Override
	public boolean isValid() {
		if(initialised)
			return max >= min ? true : false;
		else return false; 
	}

	@Override
	public String getType() {
		return "Integer Range";
	}

	@Override
	public String toString()
	{
		int range = max - min;
		return "Range(Integer:" + range + ") " + min + " - " + max;
	}
}
