package metricsOverlay.model.metrics;

import java.util.Map;

/**
 * This abstract class provides functionality
 * to create ranges that consist of named mappings.
 * 
 * @author Matthew Harward
 * @version 1/9/2009
 */
public abstract class MappedRange extends Range {
	
	/**
	 * The number of different mappings avaliable.
	 * 
	 * @return a count of the number of range mappings.
	 */
	public abstract int getNumItems();
	
	/**
	 * Returns the map containing the ranges within this 
	 * range.
	 * 
	 * @return the map.
	 */
	public abstract Map<? extends Object,String> getRangeMap();
	
	/**
	 * Returns if the name is contained as a mapping.
	 * 
	 * @param value the name.
	 * @return true if the value was found, else false.
	 */
	public abstract boolean isInRangeName(String value);
	
	/**
	 * The name of the minimum value.
	 * 
	 * @return the name.
	 */
	public abstract String getMinName();
	
	/**
	 * The name of the maximum value.
	 * 
	 * @return the name.
	 */
	public abstract String getMaxName();
	
	/**
	 * Override method of toString for mapped metrics.
	 */
	@Override
	public String toString()
	{
		return "Range(" + getType() + ":" + getNumItems() + ") " + getMinName() + "[" + getMin()
		+ "] - " + getMaxName() + "[" + getMax() + "]";
	}
	
}
