package metricsOverlay.model.metrics;

import java.util.Set;

import metricsOverlay.model.IEditable;
import metricsOverlay.model.INameable;
import metricsOverlay.model.MetricSection;

/**
 * This abstract class Metric represents a metric in the overlay form. It has
 * two subclasses, one, ActualMetric, that holds metric data and the other,
 * DerivedMetric, that acts as a proxy to allow the same metric data to be
 * displayed with different ranges.
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public abstract class Metric implements Comparable<Metric>, INameable, IEditable {

	// Fields
	protected String name;
	protected Range range;

	/**
	 * @param range
	 *            the range to set
	 */
	public void setRange(Range range) {
		this.range = range;
	}

	/**
	 * @return the range
	 */
	public Range getRange() {
		return range;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * @return the metric sections associated with this metric.
	 */
	public abstract Set<MetricSection> getMetricSections();

	@Override
	public int compareTo(Metric o) {
		if(range.equals(o.range) && name.equals(o.name))
			return 1;
		return -1;
	}

}
