package metricsOverlay.model.metrics;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * An ordered nominal range using a map.
 * 
 * List must be size >= 1
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public class OrderedNominalRange extends MappedRange {

	private Map<Integer,String> map;
	private boolean initialised;
	
	public OrderedNominalRange() {
		map = new HashMap<Integer, String>();
		initialised = false;
	}
	
	public OrderedNominalRange(Map<Integer,String> map) {
		this.map = map;
		initialised = true;
	}

	@Override
	public String getMax() {
		int maxVal = Integer.MIN_VALUE;
		for (Integer i : map.keySet())
		{
			if (i > maxVal)
				maxVal = i;
		}
		return Integer.toString(maxVal);
	}

	@Override
	public String getMin() {
		int minVal = Integer.MAX_VALUE;
		for (Integer i : map.keySet())
		{
			if (i < minVal)
				minVal = i;
		}
		return Integer.toString(minVal);
	}
	
	public String getMinName(){
		return map.get(Integer.parseInt(getMin()));
	}
	
	public String getMaxName(){
		return map.get(Integer.parseInt(getMax()));
	}

	@Override
	public boolean isInRange(String value) {
		if (map.containsKey(Integer.parseInt(value)))
			return true;
		else
			return false;
	}
	
	public boolean isInRangeName(String value) {
		if (map.containsValue(value))
			return true;
		else
			return false;
	}

	@Override
	public String getType() {
		return "Ordered Nominal Range";
	}

	@Override
	public Map<Integer, String> getRangeMap() {
		return Collections.unmodifiableMap(map);
	}

	@Override
	public int getNumItems() {
		return map.size();
	}

	@Override
	public boolean isValid() {
		// The map automatically discards identical integers.
		if(initialised)
			return map.size() > 0 ? true : false;
		else return false;
	}
}
