package metricsOverlay.model.metrics;

import metricsOverlay.model.ITyped;

/**
 * This abstract class provides a superclass of different kinds of ranges.
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public abstract class Range implements Comparable<Range>, ITyped{
	
	/**
	 * All subclasses must have a empty constructor
	 */
	public Range() {}
	
	/**
	 * @return Abstract method, should be overriden to give a String representation
	 * of this range's max value.
	 */
	public abstract String getMax();
	
	/**
	 * @return Abstract method, should be overriden to give a String representation
	 * of this range's min value.
	 */
	public abstract String getMin();
	
	/**
	 * @return Abstract method, should be overriden to detect if a provided string
	 * value is within the required range.
	 */
	public abstract boolean isInRange(String value);
	
	/**
	 * @return Abstract method, should be overriden to give a string representation
	 * of the name of the range type.
	 */
	public abstract String getType();
	
	/**
	 * Determines if the set range is valid.
	 * 
	 * @return true if valid, else false.
	 */
	public abstract boolean isValid();
	
	/**
	 * Overriden toString method
	 */
	public String toString()
	{
		return "Range(Default)";
	}
	
	/**
	 * Comparator on max and min values
	 */
	@Override
	public int compareTo(Range o) {
		if(getType().equals(o.getType()) && getMax().equals(o.getMax()) &&
				getMin().equals(o.getMin()))
			return 1;
		else return -1;
	}
}
