package metricsOverlay.model.metrics;

/**
 * An interval range using real numbers.
 * 
 * Note: This method does not check to make sure the max >= min
 * 
 * @author Matthew Harward
 * @version 15/8/2009
 */
public class RealRange extends Range {

	//Fields
	private double min;
	private double max;
	private boolean initialised;
	
	/**
	 * Parameterless constructor. Should not be called.
	 */
	public RealRange() {
		initialised = false;
	}
	
	/**
	 * Main constructor.
	 * @param min the minimum double value
	 * @param max the maximum double value
	 */
	public RealRange(double min, double max) {
		this.min = min;
		this.max = max;
		initialised = true;
	}

	@Override
	public String getMax() {
		return Double.toString(max);
	}

	@Override
	public String getMin() {
		return Double.toString(min);
	}

	@Override
	public boolean isInRange(String value) {
		Double testVal;
		try{
			testVal = Double.valueOf(value);
		} 
		catch (NumberFormatException e)
		{
			testVal = null;
		}
		boolean result = false;
		if(testVal == null)
			result = false;
		else if (testVal >= min && testVal <= max)
			result = true;
		return result;
	}
	
	@Override
	public boolean isValid() {
		if(initialised)
			return max >= min ? true : false;
		else return false; 
	}

	@Override
	public String getType() {
		return "Real Numbers Range";
	}

	@Override
	public String toString()
	{
		double range = max-min;
		return "Range(Real:" + range + ") " + min + " - " + max;
	}

}