package metricsOverlay.preferences;

import metricsOverlay.Activator;
import metricsOverlay.OverlayRegistry;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.UnderlineAugmentation;
import metricsOverlay.model.augmentation.other.ShapeDirection;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.augmentation.other.UnderlineType;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.PlatformUI;

public class AugmentationPrefController {

	private MetricFile mf;
	private AugmentationPreferencePage augPage;
	private IPreferenceStore store;
	private Augmentation curAug;

	public AugmentationPrefController(MetricFile curMf,
			AugmentationPreferencePage augPage) {
		this.mf = curMf;
		this.augPage = augPage;
		curAug = null;
		store = Activator.getDefault().getPreferenceStore();
	}

	public void updateSelector(String curSelected) {
		augPage.getAugSelector().setItems(getAugNames());
		if (curSelected == null || curSelected == ""){
			augPage.getAugSelector().clearSelection();
			curAug = null;
		}
		else {
			int i = augPage.getAugSelector().indexOf(curSelected);
			augPage.getAugSelector().select(i);
		}
	}

	public void setActiveAugmentation(Augmentation aug) {
		int a;
		if ((a = augPage.getAugSelector().indexOf(aug.getName())) > -1) {
			augPage.getAugSelector().select(a);
			augPage.getAugSelector().setText(aug.getName());
			augPage.getAugSelector().update();
			onAugSelection(aug.getName());
		}
	}

	public void onAugSelection(String selectionName) {
		curAug = mf.getMetricDocument().getAugmentation(selectionName);
		
		augPage.setEditable(curAug.canEdit());
		// System.out.println("Update: " + a.getName());
		// Set updated values
		store.setValue(AugmentationPreferencePage.AUG_NAME, curAug.getName());
		augPage.getTypeSelector().select(
				augPage.getTypeSelector().indexOf(curAug.getType()));
		store.setValue(AugmentationPreferencePage.AUG_TYPE, curAug.getType());
		augPage.setSpeciality(store
				.getString(AugmentationPreferencePage.AUG_TYPE));
		store
				.setValue(AugmentationPreferencePage.AUG_IMPORT, curAug
						.getImportance());
		store
				.setValue(AugmentationPreferencePage.AUG_ANNOTATE, curAug
						.isAnnotated());
		store.setValue(AugmentationPreferencePage.AUG_DISCRETE, curAug.isDiscrete());
		String cols = augPage.getColorEditor().createCurrentList(curAug.getColors());
		store.setValue(AugmentationPreferencePage.AUG_COLOR_EDIT, cols);

		// Speciality Combos
		if (curAug instanceof MarginAugmentation) {
			augPage.getShapeTypeSelector()
					.select(
							augPage.getShapeTypeSelector().indexOf(
									((MarginAugmentation) curAug).getShapeType()
											.toString()));
			store.setValue("shapetype", ((MarginAugmentation) curAug).getShapeType()
					.toString());
			augPage.getShapeDirSelector().select(
					augPage.getShapeDirSelector().indexOf(
							((MarginAugmentation) curAug).getShapeDirection()
									.toString()));
			store.setValue("shapedir", ((MarginAugmentation) curAug)
					.getShapeDirection().toString());
		}
		if (curAug instanceof UnderlineAugmentation) {
			augPage.getUnderlineSelector().select(
					augPage.getUnderlineSelector().indexOf(
							((UnderlineAugmentation) curAug).getUnderlineType()
									.toString()));
			store.setValue("underlinetype", ((UnderlineAugmentation) curAug)
					.getUnderlineType().toString());
		}

		// Reload all field editors
		for (FieldEditor edit : augPage.getFieldEditors()) {
			edit.load();
		}
		augPage.getColorEditor().selectItem();
	}

	protected Augmentation operateOnAugmentation(Augmentation aug) {
		aug.setName(store.getString(AugmentationPreferencePage.AUG_NAME));
		aug.setImportance(store.getInt(AugmentationPreferencePage.AUG_IMPORT));
		aug.setAnnotations(store
				.getBoolean(AugmentationPreferencePage.AUG_ANNOTATE));
		aug.setDiscrete(store
				.getBoolean(AugmentationPreferencePage.AUG_DISCRETE));
		aug.setColors(augPage.getColorEditor().getColors());

		if (aug instanceof MarginAugmentation) {
			ShapeType sType = ShapeType.valueOf(augPage.getShapeTypeSelector()
					.getText());
			ShapeDirection sDir = ShapeDirection.valueOf(augPage
					.getShapeDirSelector().getText());
			((MarginAugmentation) aug).setShape(sType, sDir);
			boolean left = true;
			if (augPage.getPositionSelector().getText().equals("Right"))
				left = false;
			((MarginAugmentation) aug).setIsLeft(left);
		}
		if (aug instanceof UnderlineAugmentation) {
			UnderlineType uType = UnderlineType.valueOf(augPage
					.getUnderlineSelector().getText());
			((UnderlineAugmentation) aug).setUnderlineType(uType);
		}
		return aug;
	}

	public String[] getAugNames() {
		String[] names = new String[0];
		if (mf != null) {
			names = new String[mf.getMetricDocument().getAllAugmentations()
					.size()];
			int count2 = 0;
			for (Augmentation mm : mf.getMetricDocument().getAllAugmentations()) {
				names[count2++] = mm.getName();
			}
		}
		return names;
	}

	public String[] getAugTypes() {
		return OverlayRegistry.getInstance().getAllAugNames();
	}

	public String[] getShapeNames() {
		ShapeType[] stype = ShapeType.values();
		String[] strArr = new String[stype.length];
		for (int i = 0; i < stype.length; i++) {
			strArr[i] = stype[i].toString();
		}
		return strArr;
	}

	public String[] getDirectionNames() {
		ShapeDirection[] sdir = ShapeDirection.values();
		String[] strArr = new String[sdir.length];
		for (int i = 0; i < sdir.length; i++) {
			strArr[i] = sdir[i].toString();
		}
		return strArr;
	}

	public String[] getUnderlineType() {
		UnderlineType[] uType = UnderlineType.values();
		String[] strArr = new String[uType.length];
		for (int i = 0; i < uType.length; i++) {
			strArr[i] = uType[i].toString();
		}
		return strArr;
	}

	public SelectionListener getComboListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				String selected = ((Combo) e.widget).getItem(((Combo) e.widget)
						.getSelectionIndex());
				onAugSelection(selected);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getTypeComboListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				String selected = ((Combo) e.widget).getItem(((Combo) e.widget)
						.getSelectionIndex());
				store.putValue(AugmentationPreferencePage.AUG_TYPE, selected);
				augPage.setSpeciality(store
						.getString(AugmentationPreferencePage.AUG_TYPE));
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		};
	}

	public SelectionListener getAddListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curAug != null){
				storeAll();
				String theName = store.getString(AugmentationPreferencePage.AUG_NAME);
				// Check for uniqueness...
				if (!mf.getMetricDocument().isAugmentationNameUnique(theName)) {
					queryDialog("Duplicate Augmentation", "You are trying" +
							" to create an augmentation with a name that already exists."
							, MessageDialog.ERROR);
				} else {
					newAugmentation();
				}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getUpdateListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curAug != null){
				storeAll();
				String theName = store.getString(AugmentationPreferencePage.AUG_NAME);
				//If the name hasn't changed
				if(curAug.getName().equals(theName)){
					updateAugmentation();
				}
				// If the name is unique, but changed
				else if (mf.getMetricDocument().isAugmentationNameUnique(theName)) {
					if(queryDialog("Name Changed", "You have changed the name of the" +
							" augmentation from \"" + curAug.getName() + "\" to \"" + theName +
							"\".\nDo you still want to update?", MessageDialog.QUESTION)){
						updateAugmentation();
					}
				} else {
					queryDialog("Name Conflict", "You have changed the name of the" +
							" augmentation to one that is not unique.\n Please change"
							+" it if you wish to update.", MessageDialog.WARNING);
				}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		};
	}

	public SelectionListener getDeleteListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curAug != null){
				storeAll();
				String theName = store.getString(AugmentationPreferencePage.AUG_NAME);
				//If the name to the current augmentation has not been changed.
				if(curAug.getName().equals(theName))
				{
					if(queryDialog("Delete Confirmation","Are you sure you wish to delete this augmentation?",
							MessageDialog.QUESTION)){
						deleteAugmentation();
					}
				}
				// Check for uniqueness... If not unique
				else if (!mf.getMetricDocument().isAugmentationNameUnique(theName)) {
					queryDialog("Delete Problem","You have changed the name of this augmentation "+
							"to \nthat of another. Please change it to something unique.",
							MessageDialog.ERROR);
				} else {
					if(queryDialog("Delete Confirmation","You have changed the name of this augmentation from \""+
							curAug.getName() + "\" to \"" + theName + "\".\n" + 
							"Are you sure you still wish to delete it?",
							MessageDialog.QUESTION)){
						deleteAugmentation();
					}
				}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		};
	}
	
	public SelectionListener getResetListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				resetPage();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	private void newAugmentation() {
		// Uses reflection to create a new instance of the right type
		String theType = store.getString(AugmentationPreferencePage.AUG_TYPE);
		Augmentation newAug = OverlayRegistry.getInstance()
			.getAugmentation(theType);
		newAug = operateOnAugmentation(newAug);
		mf.getMetricDocument().addAugmentation(newAug);
		// Then reload page...
		updateSelector(newAug.getName());
		setActiveAugmentation(newAug);
		MetricFileHandler.getInstance().update(mf,true);
		// Current page will stay loaded into the controls...
	}

	private void updateAugmentation() {
		// If the type has been changed
		if (!curAug.getType().equals(augPage.getTypeSelector().getText())) {
			queryDialog("Changed Type", "You are not allowed to change the type"
				+ " of an existing augmentation.", MessageDialog.ERROR);
		} else {
			// Call updates..
			curAug = operateOnAugmentation(curAug);
			setActiveAugmentation(curAug);
			new Runnable() {
				@Override
				public void run() {
					MetricFileHandler.getInstance().update(mf,true);
				}
			};
		}
	}

	/**
	 * 
	 */
	private void deleteAugmentation() {
		mf.getMetricDocument().removeAugmentation(
				store.getString(AugmentationPreferencePage.AUG_NAME));
			MetricFileHandler.getInstance().update(mf,true);
			resetPage();
	}

	/**
	 * 
	 */
	private void storeAll() {
		for (FieldEditor cur : augPage.getFieldEditors()) {
			cur.store();
		}
	}

	private void resetPage() {
		store.setToDefault(AugmentationPreferencePage.AUG_NAME);
		store.setToDefault(AugmentationPreferencePage.AUG_ANNOTATE);
		store.setToDefault(AugmentationPreferencePage.AUG_COLOR_EDIT);
		store.setToDefault(AugmentationPreferencePage.AUG_IMPORT);
		store.setToDefault(AugmentationPreferencePage.AUG_TYPE);
		store.setToDefault(AugmentationPreferencePage.AUG_DISCRETE);
		updateSelector(null);
		augPage.reset();
	}

	/**
	 * 
	 */
	private boolean queryDialog(String title, String contents, int type) {
		return MessageDialog.open(type, PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getShell(),
				title,contents,SWT.NONE);
	}
}
