package metricsOverlay.preferences;

import java.util.ArrayList;
import java.util.List;

import metricsOverlay.Activator;
import metricsOverlay.augmentationStrategies.AugmentationStrategy;

import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.ListEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class ColorListEditor extends ListEditor {

	int count = 1;
	private ColorFieldEditor colFieldEd;
	private int selected;
	
	public static final String COLOR_CHOICE = "cfEditor";
	
	public ColorListEditor(String name, String labelText, Composite parent){
		super(name,labelText,parent);
		GridData gLay = new GridData();
		selected = getList().getSelectionIndex();
		gLay.widthHint = 150;
		gLay.heightHint = 150;
		getList().removeAll();
		getList().setLayoutData(gLay);
		getList().addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectItem();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
	}
	
	@Override
	public Composite getButtonBoxControl(final Composite parent) {
		GridLayout gLay = new GridLayout();
		gLay.numColumns = 2;
		Composite p = new Composite(parent, SWT.NONE);
		p.setLayout(gLay);
		colFieldEd = new ColorFieldEditor(COLOR_CHOICE, "Current Color:", p);
		colFieldEd.getColorSelector().setColorValue(new RGB(255,0,0));
		colFieldEd.setPropertyChangeListener(new IPropertyChangeListener() {
			
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateList();
			}
		});
		colFieldEd.load();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		colFieldEd.setPreferenceStore(store);
		Composite ret = super.getButtonBoxControl(p);
		return ret;
	}
	
	@Override
	protected String createList(String[] items) {
		String finalStr = "";
		for(String st : items){
			finalStr += st + "@";
		}
		return finalStr.length() > 0 ? finalStr.substring(0, finalStr.length() - 1) : "";
	}
	
	protected String generateString(RGB rgb){
		return "Color [" + rgb.red+", "+rgb.green+", " +rgb.blue + "]";
	}

	@Override
	protected String getNewInputObject() {
		RGB curColor = colFieldEd.getColorSelector().getColorValue();
		colFieldEd.store();
		String listStr = generateString(curColor);
		return listStr;
	}

	@Override
	protected String[] parseString(String stringList) {
		return stringList.split("@");
	}

	/**
	 * 
	 */
	protected void updateList() {
		if(getList().getSelectionCount() > 0){
			colFieldEd.store();
			RGB curColor = colFieldEd.getColorSelector().getColorValue();
			String newString = generateString(curColor);
			getList().setItem(getList().getSelectionIndex(), newString);
		}
	}

	public String createCurrentList(List<Color> colors) {
		getList().removeAll();
		String[] stArr = new String[colors.size()];
		for(int i = 0; i < colors.size(); i++){
			String curString = generateString(colors.get(i).getRGB());
			stArr[i] = curString;
		}
		return createList(stArr);
	}

	public void selectItem() {
		if(selected == getList().getSelectionIndex()){
			getList().deselectAll();
			getUpButton().setEnabled(false);
			getDownButton().setEnabled(false);
			getRemoveButton().setEnabled(false);
			selected = -1;
		} else {
			//getList().select(i);
			selected = getList().getSelectionIndex();
		}
		selectListItem();
	}

	/**
	 * 
	 */
	protected void selectListItem() {
		String selItem = getList().getItem(0);
		if(getList().getSelection().length > 0){
			selItem = getList().getSelection()[0];
			RGB rgb = AugmentationStrategy.parseColor(selItem).getRGB();
			colFieldEd.getColorSelector().setColorValue(rgb);
		}
	}

	public List<Color> getColors() {
		List<Color> curL = new ArrayList<Color>();
		for(String s : getList().getItems()){
			curL.add(AugmentationStrategy.parseColor(s));
		}
		return curL;
	}

}
