package metricsOverlay.preferences;

import java.util.HashMap;
import java.util.Map;

import metricsOverlay.Activator;
import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.IntegerRange;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.ListEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class CompositeRangeEditor extends ListEditor {

	int count = 1;
	private StringFieldEditor strFieldEd;
	private IntegerFieldEditor intFieldMin;
	private IntegerFieldEditor intFieldMax;
	private Map<IntegerRange,String> rangeMap;
	private Map<String,IntegerRange> rangeTracker; //Holds the list's textual descrip
	private int selectionIndex;
	
	public static final String RANGE_NAME = "rangeName";
	public static final String RANGE_MIN = "rangeMin";
	public static final String RANGE_MAX = "rangeMax";
	private static final String UPDATE = "update";
	private static final String ADD = "add";
	
	public CompositeRangeEditor(String name, String labelText, Composite parent){
		super(name,labelText,parent);
		GridData gLay = new GridData();
		rangeMap = new HashMap<IntegerRange, String>();
		rangeTracker = new HashMap<String,IntegerRange>();
		selectionIndex = getList().getSelectionIndex();
		//getUpButton().setVisible(false);
		//getDownButton().setVisible(false);
		//Can be uncommented when we have auto ordering...
		gLay.widthHint = 150;
		gLay.heightHint = 150;
		getList().removeAll();
		getList().setLayoutData(gLay);
		getList().addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectItem();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
	}
	
	@Override
	public Composite getButtonBoxControl(final Composite parent) {
		GridLayout gLay = new GridLayout();
		gLay.numColumns = 2;
		Composite p = new Composite(parent, SWT.NONE);
		p.setLayout(gLay);
		strFieldEd = new StringFieldEditor(RANGE_NAME, "Range Name:", p);
		//strFieldEd.setEmptyStringAllowed(false);
		//strFieldEd.setStringValue("Item");
		strFieldEd.setPropertyChangeListener(new IPropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateList();
			}
		});
		strFieldEd.load();
		//strFieldEd.getLabelControl(p).setLayoutData(
			//	new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		intFieldMin = new IntegerFieldEditor(RANGE_MIN, "Minimum:", p);
		intFieldMin.setStringValue("1");
		intFieldMin.setPropertyChangeListener(new IPropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateList();
			}
		});
		intFieldMin.load();
		intFieldMax = new IntegerFieldEditor(RANGE_MAX, "Maximum:", p);
		intFieldMax.setStringValue("1");
		intFieldMax.setPropertyChangeListener(new IPropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateList();
			}
		});
		intFieldMax.load();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		strFieldEd.setPreferenceStore(store);
		intFieldMax.setPreferenceStore(store);
		intFieldMin.setPreferenceStore(store);
		Composite ret = super.getButtonBoxControl(p);
		//ret.pack();
		return ret;
	}
	
	@Override
	protected String createList(String[] items) {
		String finalStr = "";
		for(String st : items){
			finalStr += st + "@";
		}
		return finalStr.length() > 0 ? finalStr.substring(0, finalStr.length() - 1) : "";
	}
	
	protected String generateString(String val, int min, int max){
		return val + " : [" + min + ", " + max + "]";
	}

	@Override
	protected String getNewInputObject() {
		if(validInput(ADD)){
		strFieldEd.store();
		intFieldMax.store();
		intFieldMin.store();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		String curName = store.getString(RANGE_NAME);
		String listStr = generateString(curName, store.getInt(RANGE_MIN),
				store.getInt(RANGE_MAX));
		IntegerRange newIR = new IntegerRange(store.getInt(RANGE_MIN),
				store.getInt(RANGE_MAX));
		rangeMap.put(newIR,curName);
		rangeTracker.put(listStr,newIR);
		return listStr;
		} else return null;
	}
	
	private boolean validInput(String input){
		if(!(intFieldMax.isValid() && intFieldMin.isValid())){
			showErrorMessage("The maximum and minimum boxes must be integers.");
			return false;
		}else if(!strFieldEd.isValid()){
			showErrorMessage("You must name this range item.");
			return false;
		}else{
		strFieldEd.store();
		intFieldMax.store();
		intFieldMin.store();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		if(store.getInt(RANGE_MIN) > store.getInt(RANGE_MAX))
		{
			showErrorMessage("The minimum value must be >= to the maximum value.");
			return false;
		}
		Map<IntegerRange, String> rangeClone = new HashMap<IntegerRange,String>(rangeMap);
		String curSelected = null;
		if(input.equals(UPDATE) && getList().getSelectionIndex() > -1 &&(curSelected = getList().getItem(getList().getSelectionIndex())) != null)
			rangeClone.remove(rangeTracker.get(curSelected));
		CompositeIntegerRange testRange = new CompositeIntegerRange(rangeClone);
		for(int i = store.getInt(RANGE_MIN); i <= store.getInt(RANGE_MAX); i++){
			if(testRange.isInRange(String.valueOf(i))){
				showErrorMessage("This range overlaps with another range");
				return false;
			}
		}
		}
		clearErrorMessage();
		return true;
	}

	@Override
	protected String[] parseString(String stringList) {
		return stringList.split("@");
	}

	/**
	 * 
	 */
	protected void updateList() {
		if(validInput(UPDATE) && getList().getSelectionCount() > 0){
			strFieldEd.store();
			intFieldMax.store();
			intFieldMin.store();
			IPreferenceStore store = Activator.getDefault().getPreferenceStore();
			String curName = store.getString(RANGE_NAME);
			int intMinVal = store.getInt(RANGE_MIN);
			int intMaxVal = store.getInt(RANGE_MAX);
			String newString = generateString(curName, intMinVal, intMaxVal);
			//Then replace existing item
			String oldString = getList().getItem(getList().getSelectionIndex());
			IntegerRange oldIR = rangeTracker.get(oldString);
			rangeTracker.remove(oldString);
			rangeMap.remove(oldIR);
			IntegerRange newIR = new IntegerRange(intMinVal,intMaxVal);
			rangeTracker.put(newString,newIR);
			rangeMap.put(newIR,curName);
			getList().setItem(getList().getSelectionIndex(), newString);
		}
	}

	public String createCurrentList(CompositeIntegerRange cr) {
		rangeMap.clear();
		rangeTracker.clear();
		getList().removeAll();
		if(cr != null)
			rangeMap = cr.getRangeMap();
		String[] stArr = new String[rangeMap.size()];
		int count = 0;
		for(IntegerRange ir : rangeMap.keySet()){
			String curString = generateString(rangeMap.get(ir),
					Integer.valueOf(ir.getMin()),Integer.valueOf(ir.getMax()));
			rangeTracker.put(curString, ir);
			stArr[count++] = curString;
		}
		return createList(stArr);
	}

	public void selectItem() {
		if(selectionIndex == getList().getSelectionIndex()){
			getList().deselectAll();
			getUpButton().setEnabled(false);
			getDownButton().setEnabled(false);
			getRemoveButton().setEnabled(false);
			selectionIndex = -1;
		} else {
			selectionIndex = getList().getSelectionIndex();
		}
		selectListItem();
	}

	/**
	 * 
	 */
	protected void selectListItem() {
		String selItem = getList().getItem(0);
		if(getList().getSelection().length > 0) {
			selItem = getList().getSelection()[0];
			IntegerRange ir = rangeTracker.get(selItem);
			if(ir != null){
			strFieldEd.setStringValue(rangeMap.get(ir));
			intFieldMin.setStringValue(ir.getMin());
			intFieldMax.setStringValue(ir.getMax());
			}
		}
	}

	public CompositeIntegerRange getNewCompositeRange() {
		return new CompositeIntegerRange(rangeMap);
	}

}
