package metricsOverlay.preferences;

import metricsOverlay.Activator;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;

/**
 * This provides the code in-front for the main
 * introductory preference page.
 * 
 * @author Matthew Harward
 * @version 26/8/2009
 */
public class MetricsOverlayPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public static final String PRESERVE_SETTINGS = "preserveSettings";
	public static final String USE_DEFAULTS = "usedefaults";
	
	public MetricsOverlayPreferencePage() {
		super(GRID);
		setTitle("Metrics Overlay Preferences");
		setDescription("Specify your preferences for metrics decoration" + 
				" by choosing one of the three pages below:");
		//setImageDescriptor(ImageDescriptor.createFromImage(new Image(this.getShell().getDisplay(),"icons/mlogo.jpg")));
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		//setDescription("Select color coding colors.");
		
		
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		Composite parent = getFieldEditorParent();
		FormToolkit toolkit = new FormToolkit(parent.getDisplay());
		Form dform = toolkit.createForm(parent);
		dform.setText("Augmentations");
		dform.setBackground(parent.getBackground());
		GridLayout dlayout = new GridLayout();
		dform.getBody().setLayout(dlayout);
		Label dLab = new Label(dform.getBody(),SWT.None);
		dLab.setText("Set and create the augmentation properties of avaliable overlays.");
		String durl = "Augmentation Preference Page";
		Hyperlink dlink = toolkit.createHyperlink(dform.getBody(), durl, SWT.WRAP);
		dlink.setBackground(parent.getBackground());
		dlink.addHyperlinkListener(new HyperlinkAdapter() {
		     public void linkActivated(HyperlinkEvent e) {
		                PreferenceDialog diag = org.eclipse.ui.dialogs.PreferencesUtil.createPreferenceDialogOn(
								PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
								"metricsOverlay.preferences.AugmentationPreferencePage",
								null, null);
						diag.open();	
		     }
		});
		///
		Form mapform = toolkit.createForm(parent);
		mapform.setText("Mappings");
		mapform.setBackground(parent.getBackground());
		GridLayout maplayout = new GridLayout();
		mapform.getBody().setLayout(maplayout);
		Label mapLab = new Label(mapform.getBody(),SWT.None);
		mapLab.setText("Set up mappings between different metrics and overlays. \nChange " +
				"the interpolation between these elements.");
		String mapurl = "Mapping Preference Page";
		Hyperlink maplink = toolkit.createHyperlink(mapform.getBody(), mapurl, SWT.WRAP);
		maplink.setBackground(parent.getBackground());
		maplink.addHyperlinkListener(new HyperlinkAdapter() {
		     public void linkActivated(HyperlinkEvent e) {
		                PreferenceDialog diag = org.eclipse.ui.dialogs.PreferencesUtil.createPreferenceDialogOn(
								PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
								"metricsOverlay.preferences.MappingPreferencePage",
								null, null);
						diag.open();	
		     }
		});
		///
		Form form = toolkit.createForm(parent);
		form.setText("Metrics");
		form.setBackground(parent.getBackground());
		GridLayout layout = new GridLayout();
		form.getBody().setLayout(layout);
		Label lab = new Label(form.getBody(),SWT.None);
		lab.setText("Set up the properties of existing metrics.");
		String url = "Metrics Preference Page";
		Hyperlink link = toolkit.createHyperlink(form.getBody(), url, SWT.WRAP);
		link.setBackground(parent.getBackground());
		link.addHyperlinkListener(new HyperlinkAdapter() {
		     public void linkActivated(HyperlinkEvent e) {
		                PreferenceDialog diag = org.eclipse.ui.dialogs.PreferencesUtil.createPreferenceDialogOn(
								PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
								"metricsOverlay.preferences.MetricsPreferencePage",
								null, null);
						diag.open();	
		     }
		});
		addField(new SpacerFieldEditor(parent));
		addField(new BooleanFieldEditor(PRESERVE_SETTINGS ,"Keep identical settings between open files.", parent));
		addField(new BooleanFieldEditor(USE_DEFAULTS ,"Provide default augmentations", parent));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		new PreferenceInitializer().initializeDefaultPreferences();
	}
}
