package metricsOverlay.preferences;

import metricsOverlay.Activator;
import metricsOverlay.OverlayRegistry;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.DerivedMetric;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.Metric;
import metricsOverlay.model.metrics.Range;
import metricsOverlay.model.metrics.RealRange;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.PlatformUI;

public class MetricsPrefController {

	private MetricFile curMf;
	private MetricsPreferencePage mPref;
	private IPreferenceStore store;
	private Metric curMet;

	public MetricsPrefController(MetricFile mf,
			MetricsPreferencePage metricsPreferencePage) {
		curMf = mf;
		mPref = metricsPreferencePage;
		store = Activator.getDefault().getPreferenceStore();
		curMet = null;
	}

	public String[] getMetNames() {
		String[] names = new String[0];
		if (curMf != null) {
			names = new String[curMf.getMetricDocument().getAllMetrics()
					.size()];
			int count2 = 0;
			for (Metric mm : curMf.getMetricDocument().getAllMetrics()) {
				names[count2++] = mm.getName();
			}
		}
		return names;
	}

	public String[] getRangeTypes() {
		return OverlayRegistry.getInstance().getAllRangeNames();
	}

	private DerivedMetric createNewMetric() {
		//Create range
		IntegerRange range = new IntegerRange();//TODO
		
		String derMetName = mPref.getDerivedMetSelector().getText();
		String theName = store.getString(MetricsPreferencePage.MET_NAME);
		DerivedMetric m = new DerivedMetric(theName, 
				range,(ActualMetric)curMf.getMetricDocument().getMetric(derMetName));
		return m;
	}

	private Metric operateOnMetric(Metric met) {
		// TODO Auto-generated method stub
		return null;
	}

	public void updateSelector(String curSelected) {
		mPref.getMetSelector().setItems(getMetNames());
		if (curSelected == null || curSelected == ""){
			mPref.getMetSelector().clearSelection();
			curMet = null;
		}
		else {
			int i = mPref.getMetSelector().indexOf(curSelected);
			mPref.getMetSelector().select(i);
		}
	}

	protected void onMetSelection(String selected) {
		curMet = curMf.getMetricDocument().getMetric(selected);
		
		mPref.setEditable(curMet.canEdit());
		
		boolean derived = curMet instanceof DerivedMetric;
		// System.out.println("Update: " + a.getName());
		// Set updated values
		store.setValue(MetricsPreferencePage.MET_NAME, curMet.getName());

		// Speciality Combos
		if(derived){
			String from = ((DerivedMetric)curMet).derivedFrom();
			mPref.getDerivedMetSelector().setText(from);
		} else {
			mPref.getDerivedMetSelector().clearSelection();
		}
		mPref.getRangeSelector().setText(curMet.getRange().getType());
		
		Range ran = curMet.getRange();
		mPref.setSpeciality(ran.getType());
		if(ran.getType().equals(new IntegerRange().getType())){
			store.setValue(MetricsPreferencePage.MET_INTMIN, 
					Integer.valueOf(ran.getMin()));
			store.setValue(MetricsPreferencePage.MET_INTMAX, 
					Integer.valueOf(ran.getMin()));
		} else if(ran.getType().equals(new RealRange().getType())){
			store.setValue(MetricsPreferencePage.MET_REALMIN, 
					ran.getMin());
			store.setValue(MetricsPreferencePage.MET_REALMAX, 
					ran.getMin());
		} else {
			//Ordered Nominal
		}
		
		if(!derived){
			//Disable all field editors in this area
		}

		// Reload all field editors
		for (FieldEditor edit : mPref.getFieldEditors()) {
			edit.load();
		}
	}

	public SelectionListener getComboListener() {
		return new SelectionListener() {
	
			@Override
			public void widgetSelected(SelectionEvent e) {
				String selected = ((Combo) e.widget).getItem(((Combo) e.widget)
						.getSelectionIndex());
				onMetSelection(selected);
			}
	
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getAddListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curMet != null){
					storeAll();
					String theName = store.getString(MetricsPreferencePage.MET_NAME);
					// Check for uniqueness...
					if (!curMf.getMetricDocument().isMetricNameUnique(theName)) {
						queryDialog("Duplicate Metric", "You are trying" +
								" to create an metric with a name that already exists."
								, MessageDialog.ERROR);
					} else {
						newMetric();
					}
					}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getUpdateListener() {
		return new SelectionListener() {
	
			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curMet != null){
					storeAll();
					String theName = store.getString(MetricsPreferencePage.MET_NAME);
					//If the name hasn't changed
					if(curMet.getName().equals(theName)){
						updateMetric();
					}
					// If the name is unique, but changed
					else if (curMf.getMetricDocument().isMetricNameUnique(theName)) {
						if(queryDialog("Name Changed", "You have changed the name of the" +
								" metric from \"" + curMet.getName() + "\" to \"" + theName +
								"\".\nDo you still want to update?", MessageDialog.QUESTION)){
							updateMetric();
						}
					} else {
						queryDialog("Name Conflict", "You have changed the name of the" +
								" metric to one that is not unique.\n Please change"
								+" it if you wish to update.", MessageDialog.WARNING);
					}
					}
			}
	
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getDeleteListener() {
		return new SelectionListener() {
	
			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curMet != null){
					storeAll();
					String theName = store.getString(MetricsPreferencePage.MET_NAME);
					//If the name to the current metric has not been changed.
					if(curMet.getName().equals(theName))
					{
						if(queryDialog("Delete Confirmation","Are you sure you wish to delete this metric?",
								MessageDialog.QUESTION)){
							deleteMetric();
						}
					}
					// Check for uniqueness... If not unique
					else if (!curMf.getMetricDocument().isMetricNameUnique(theName)) {
						queryDialog("Delete Problem","You have changed the name of this metric "+
								"to \nthat of another. Please change it to something unique.",
								MessageDialog.ERROR);
					} else {
						if(queryDialog("Delete Confirmation","You have changed the name of this metric from \""+
								curMet.getName() + "\" to \"" + theName + "\".\n" + 
								"Are you sure you still wish to delete it?",
								MessageDialog.QUESTION)){
							deleteMetric();
						}
					}
					}
			}
	
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getResetListener() {
		return new SelectionListener() {
	
			@Override
			public void widgetSelected(SelectionEvent e) {
				resetPage();
			}
	
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	private void deleteMetric() {
		curMf.getMetricDocument().removeMetric(
				store.getString(MetricsPreferencePage.MET_NAME));
			MetricFileHandler.getInstance().update(curMf,true);
			resetPage();
	}

	private void resetPage() {
		store.setToDefault(MetricsPreferencePage.MET_NAME);
		//TODO To be completed...
		updateSelector(null);
		mPref.reset();
	}

	public void setActiveMetric(Metric metric) {
		int a;
		if ((a = mPref.getMetSelector().indexOf(metric.getName())) > -1) {
			mPref.getMetSelector().select(a);
			mPref.getMetSelector().setText(metric.getName());
			mPref.getMetSelector().update();
			onMetSelection(metric.getName());
		}
	}

	public SelectionListener getDerivedComboListener() {
		return new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				// TODO Auto-generated method stub	
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {}
		};
	}

	public SelectionListener getRangeComboListener() {
		return new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				mPref.setSpeciality(mPref.getRangeSelector().getText());
				if(curMet.getRange().getType().equals(new CompositeIntegerRange().getType()))
					mPref.getCompositeRangeEditor().
					createCurrentList((CompositeIntegerRange) curMet.getRange());
				else
					mPref.getCompositeRangeEditor().createCurrentList(null);
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {}
		};
	}

	private void newMetric() {
		DerivedMetric newMet = createNewMetric();
		curMet = newMet;
		curMf.getMetricDocument().addMetric(curMet);
		// Then reload page...
		updateSelector(curMet.getName());
		setActiveMetric(curMet);
		MetricFileHandler.getInstance().update(curMf,true);
		// Current page will stay loaded into the controls...
	}

	private void updateMetric() {
		// Call updates..
		curMet = operateOnMetric(curMet);
		setActiveMetric(curMet);
		MetricFileHandler.getInstance().update(curMf,true);
	}

	/**
	 * 
	 */
	private boolean queryDialog(String title, String contents, int type) {
		return MessageDialog.open(type, PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getShell(),
				title,contents,SWT.NONE);
	}

	private void storeAll() {
		for (FieldEditor cur : mPref.getFieldEditors()) {
			cur.store();
		}
	}
}
