package metricsOverlay.preferences;

import java.util.HashMap;
import java.util.Map;

import metricsOverlay.Activator;
import metricsOverlay.model.metrics.OrderedNominalRange;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.ListEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class OrderedNominalEditor extends ListEditor {

	int count = 1;
	private StringFieldEditor strFieldEd;
	private IntegerFieldEditor intField;
	private Map<Integer,String> rangeMap;
	private Map<String,Integer> rangeTracker; //Holds the list's textual descrip
	private int selectionIndex;
	
	public static final String RANGE_NAME = "ONrangeName";
	public static final String RANGE_VAL = "ONrangeVal";
	private static final String UPDATE = "update";
	private static final String ADD = "add";
	
	public OrderedNominalEditor(String name, String labelText, Composite parent){
		super(name,labelText,parent);
		GridData gLay = new GridData();
		rangeMap = new HashMap<Integer, String>();
		rangeTracker = new HashMap<String,Integer>();
		selectionIndex = getList().getSelectionIndex();
		//getUpButton().setVisible(false);
		//getDownButton().setVisible(false);
		//Can be uncommented when we have auto ordering...
		gLay.widthHint = 150;
		gLay.heightHint = 150;
		getList().removeAll();
		getList().setLayoutData(gLay);
		getList().addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectItem();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
	}
	
	@Override
	public Composite getButtonBoxControl(final Composite parent) {
		GridLayout gLay = new GridLayout();
		gLay.numColumns = 2;
		Composite p = new Composite(parent, SWT.NONE);
		p.setLayout(gLay);
		strFieldEd = new StringFieldEditor(RANGE_NAME, "Range Name:", p);
		//strFieldEd.setEmptyStringAllowed(false);
		//strFieldEd.setStringValue("Item");
		strFieldEd.setPropertyChangeListener(new IPropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateList();
			}
		});
		strFieldEd.load();
		//strFieldEd.getLabelControl(p).setLayoutData(
			//	new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		intField = new IntegerFieldEditor(RANGE_VAL, "Range Value:", p);
		intField.setStringValue("1");
		intField.setPropertyChangeListener(new IPropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateList();
			}
		});
		intField.load();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		strFieldEd.setPreferenceStore(store);
		intField.setPreferenceStore(store);
		Composite ret = super.getButtonBoxControl(p);
		//ret.pack();
		return ret;
	}
	
	@Override
	protected String createList(String[] items) {
		String finalStr = "";
		for(String st : items){
			finalStr += st + "@";
		}
		return finalStr.length() > 0 ? finalStr.substring(0, finalStr.length() - 1) : "";
	}
	
	protected String generateString(String val, int min){
		return val + " : [ " + min + " ]";
	}

	@Override
	protected String getNewInputObject() {
		if(validInput(ADD)){
		strFieldEd.store();
		intField.store();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		String curName = store.getString(RANGE_NAME);
		String listStr = generateString(curName, store.getInt(RANGE_VAL));
		rangeMap.put(store.getInt(RANGE_VAL),curName);
		rangeTracker.put(listStr,store.getInt(RANGE_VAL));
		return listStr;
		} else return null;
	}
	
	private boolean validInput(String input){
		if(!(intField.isValid())){
			showErrorMessage("The range value box must be an integer.");
			return false;
		}else if(!strFieldEd.isValid()){
			showErrorMessage("You must name this range item.");
			return false;
		}else{
		strFieldEd.store();
		intField.store();
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		Map<Integer, String> rangeClone = new HashMap<Integer,String>(rangeMap);
		String curSelected = null;
		if(input.equals(UPDATE) && getList().getSelectionIndex() > -1 &&(curSelected = getList().getItem(getList().getSelectionIndex())) != null)
			rangeClone.remove(rangeTracker.get(curSelected));
		if(rangeClone.containsKey(store.getInt(RANGE_VAL))){
			showErrorMessage("A range item with the entered value already exists");
			return false;
		}
		}
		clearErrorMessage();
		return true;
	}

	@Override
	protected String[] parseString(String stringList) {
		return stringList.split("@");
	}

	/**
	 * 
	 */
	protected void updateList() {
		if(validInput(UPDATE) && getList().getSelectionCount() > 0){
			strFieldEd.store();
			intField.store();
			IPreferenceStore store = Activator.getDefault().getPreferenceStore();
			String curName = store.getString(RANGE_NAME);
			int intVal = store.getInt(RANGE_VAL);
			String newString = generateString(curName, intVal);
			//Then replace existing item
			String oldString = getList().getItem(getList().getSelectionIndex());
			Integer oldInt = rangeTracker.get(oldString);
			rangeTracker.remove(oldString);
			rangeMap.remove(oldInt);
			rangeTracker.put(newString,intVal);
			rangeMap.put(intVal,curName);
			getList().setItem(getList().getSelectionIndex(), newString);
		}
	}

	public String createCurrentList(OrderedNominalRange onr) {
		rangeMap.clear();
		rangeTracker.clear();
		getList().removeAll();
		if(onr != null)
			rangeMap = onr.getRangeMap();
		String[] stArr = new String[rangeMap.size()];
		int count = 0;
		for(Integer integer : rangeMap.keySet()){
			String curString = generateString(rangeMap.get(integer),integer);
			rangeTracker.put(curString, integer);
			stArr[count++] = curString;
		}
		return createList(stArr);
	}

	public void selectItem() {
		if(selectionIndex == getList().getSelectionIndex()){
			getList().deselectAll();
			getUpButton().setEnabled(false);
			getDownButton().setEnabled(false);
			getRemoveButton().setEnabled(false);
			selectionIndex = -1;
		} else {
			selectionIndex = getList().getSelectionIndex();
		}
		selectListItem();
	}

	/**
	 * 
	 */
	protected void selectListItem() {
		String selItem = getList().getItem(0);
		if(getList().getSelection().length > 0) {
			selItem = getList().getSelection()[0];
			Integer integer = rangeTracker.get(selItem);
			if(integer != null){
			strFieldEd.setStringValue(rangeMap.get(integer));
			intField.setStringValue(String.valueOf(integer));
			}
		}
	}

	public OrderedNominalRange getNewOrderedNominalRange() {
		return new OrderedNominalRange(rangeMap);
	}

}
