package metricsOverlay.views;

import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;

/**
 * This class provides functionality of the click of a table cell by
 * implements an ICellModifier. An ICellModifier is called when the user 
 * modifies a cell in the tableViewer.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class ActiveMappingCellModifier implements ICellModifier {
	private ActiveMappingView mappingTableViewer;
	
	/**
	 * Constructor.
	 * 
	 * @param mappingTableViewer the current <code>ActiveMappingView</code>.
	 */
	public ActiveMappingCellModifier(ActiveMappingView mappingTableViewer) {
		super();
		this.mappingTableViewer = mappingTableViewer;
	}

	/**
	 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object, java.lang.String)
	 */
	public boolean canModify(Object element, String property) {
		return true;
	}

	/**
	 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object, java.lang.String)
	 */
	public Object getValue(Object element, String property) {

		// Find the index of the column
		int columnIndex = mappingTableViewer.getColumnNames().indexOf(property);

		Object result = null;
		Mapping map = (Mapping) element;

		switch (columnIndex) {
			case 0 : // ACTIVE_COLUMN 
				result = new Boolean(map.getMappingState().equals(MappingState.ActiveMapping));
				break;
			case 1 : // MAPPING_COLUMN 
				result = map.getName();
				break;
			case 2 : // METRIC_COLUMN 
				result = map.getMetric().getName();			
				break;
			case 3 : // DISPLAY_COLUMN 
				result = map.getAugmentation().getName();
				break;
			case 4: //COUNT_COLUMN
				result = map.getMetric().getMetricSections().size();
				break;
			default :
				result = "";
		}
		return result;	
	}

	/**
	 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object, java.lang.String, java.lang.Object)
	 */
	public void modify(Object element, String property, Object value) {

		// Find the index of the column 
		int columnIndex	= mappingTableViewer.getColumnNames().indexOf(property);
			
		TableItem item = (TableItem) element;
		if(item != null) {
		Mapping map = (Mapping) item.getData();
		String valueString;

		switch (columnIndex) {
			case 0 : // ACTIVE_COLUMN 
			    if(Boolean.valueOf(String.valueOf(value)))
			    	map.setMappingState(MappingState.ActiveMapping);
			    else
			    	map.setMappingState(MappingState.DeclaredMapping);
			    mappingTableViewer.updateModel(map);
			    //MetricFileHandler.getInstance().forceUpdate();
				break;
			case 1 : // MAPPING_COLUMN 
				valueString = String.valueOf(value);
				map.setName(valueString); //Should we allow??
				break;
			case 2 : // METRIC_COLUMN 
				valueString = String.valueOf(value);
				map.getMetric().setName(valueString); //Note this just changes the name, not actual metric..
				break;
			case 3 : // AUGMENTATION_COLUMN
				valueString = String.valueOf(value);
				map.getAugmentation().setName(valueString);
				break;
			case 4 :
			case 5:
			default :
			}
		mappingTableViewer.getTaskList().mappingChanged(map);
		}
	}
}
