package metricsOverlay.views;

import metricsOverlay.Activator;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;


/**
 * This class provides the labels of the cells in the table viewer.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class ActiveMappingLabelProvider 
	extends LabelProvider
	implements ITableLabelProvider {

	// Names of images used to represent checkboxes
	public static final String CHECKED_IMAGE 	= "icons/checked.gif";
	public static final String UNCHECKED_IMAGE  = "icons/unchecked.gif";
	public static final String EDIT_IMAGE = "icons/smalledit.gif";
	public static final String REMOVE_IMAGE = "icons/smallremove.gif";

	
	/**
	 * Returns the image with the given key, or <code>null</code> if not found.
	 */
	private Image getImage(boolean isSelected) {
		String key = isSelected ? CHECKED_IMAGE : UNCHECKED_IMAGE;
		//PlatformUI.getWorkbench().getSharedImages().getImage()
		return Activator.getDefault().getImageRegistry().get(key);
	}
	
	public Image getEditImage() {
		return Activator.getDefault().getImageRegistry().get(EDIT_IMAGE);
	}
	
	private Image getRemoveImage() {
		return Activator.getDefault().getImageRegistry().get(REMOVE_IMAGE);
	}

	/**
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
	 */
	public String getColumnText(Object element, int columnIndex) {
		String result = "";
		Mapping map = (Mapping) element;
		switch (columnIndex) {
			case 0:  // COMPLETED_COLUMN
				break;
			case 1 :
				result = map.getName();
				break;
			case 2 :
				if(map.isUsable())
					result = map.getMetric().getName();
				break;
			case 3 :
				if(map.isUsable())
					result = map.getAugmentation().getName();
				break;
			case 4 :
				if(map.isUsable())
					result = String.valueOf(map.getMetric().getMetricSections().size());
				break; 
			case 5 :
				result = "Remove";
				break;
			default :
				break; 	
		}
		return result;
	}

	/**
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
	 */
	public Image getColumnImage(Object element, int columnIndex) {
		if (columnIndex == 0)   // ACTIVE_COLUMN?
			return getImage(((Mapping) element).getMappingState().equals(MappingState.ActiveMapping));
		else if (columnIndex == 5)
			return getRemoveImage();
		else
			return null;
	}

}
