/**
 * (c) Copyright Mirasol Op'nWorks Inc. 2002, 2003. 
 * http://www.opnworks.com
 * Created on Apr 2, 2003 by lgauthier@opnworks.com
 * 
 */

package metricsOverlay.views;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;

/**
 * This class keeps track of the current list of mappings in the table view.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class ActiveMappingList {

	private List<Mapping> mappings = new ArrayList<Mapping>();
	private Set<IActiveMappingListViewer> changeListeners = new HashSet<IActiveMappingListViewer>();
	
	/**
	 * Constructor
	 */
	public ActiveMappingList(MetricDocument mDoc) {
		super();
		this.initData(mDoc);
	}
	
	/*
	 * Initialize the table data.
	 * Create COUNT tasks and add them them to the 
	 * collection of tasks
	 */
	private void initData(MetricDocument mDoc) {
		if(mDoc != null){
		for(Mapping mm : mDoc.getAllMappings()){
			if(!mm.getMappingState().equals(MappingState.StubMapping))
				mappings.add(mm);
		}
		}
	}
	
	/**
	 * Return the collection of tasks
	 */
	public List<Mapping> getMappingList() {
		return mappings;
	}
	
	/**
	 * Add a new task to the collection of tasks
	 */
	public void addMapping() {/** This should not be possible
		Mapping map = new Mapping("New task");
		tasks.add(tasks.size(), task);
		Iterator iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IMappingListViewer) iterator.next()).addTask(task);*/
	}

	/**
	 * @param map
	 */
	public void removeMapping(Mapping map) {
		mappings.remove(map);
		Iterator<IActiveMappingListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IActiveMappingListViewer) iterator.next()).removeMapping(map);
	}

	/**
	 * @param map
	 */
	public void mappingChanged(Mapping map) {
		Iterator<IActiveMappingListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IActiveMappingListViewer) iterator.next()).updateMapping(map);
	}

	/**
	 * @param viewer
	 */
	public void removeChangeListener(IActiveMappingListViewer viewer) {
		changeListeners.remove(viewer);
	}

	/**
	 * @param viewer
	 */
	public void addChangeListener(IActiveMappingListViewer viewer) {
		changeListeners.add(viewer);
	}

}
