
package metricsOverlay.views;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

/**
 * This class provides sorting functionality to the tableviewer.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class ActiveMappingSorter extends ViewerSorter {

	//Static values
	public static final int ACTIVE = 1;
	public static final int MAPPING = 2;
	public static final int METRIC = 3;
	public static final int AUGMENT = 4;
	public static final int COUNT = 5;
	
	public static final boolean DESC = true;
	public static final boolean ASC = false;

	// Criteria that the instance uses 
	private int criteria;
	private boolean desc;

	/**
	 * Creates a resource sorter that will use the given sort criteria.
	 *
	 * @param criteria the sort criterion to use: one of <code>NAME</code> or 
	 *   <code>TYPE</code>
	 */
	public ActiveMappingSorter(int criteria, boolean desc) {
		super();
		this.criteria = criteria;
		this.desc = desc;
	}

	/* (non-Javadoc)
	 * Method declared on ViewerSorter.
	 */
	public int compare(Viewer viewer, Object o1, Object o2) {

		Mapping map1 = (Mapping) o1;
		Mapping map2 = (Mapping) o2;
		
		if(desc == false)
		{
			Mapping temp = map2;
			map2 = map1;
			map1 = temp;
		}

		switch (criteria) {
			case ACTIVE :
				return compareActive(map1, map2);
			case MAPPING :
				return compareMapping(map1, map2);
			case METRIC :
				return compareMetric(map1, map2);
			case AUGMENT :
				return compareDisplay(map1, map2);
			case COUNT :
				return compareCount(map1, map2);
			default:
				return 0;
		}
	}

	private int compareCount(Mapping map1, Mapping map2) {
		if(map1.getMetric().getMetricSections().size() > 
			map2.getMetric().getMetricSections().size())
			return 1;
		else if(map1.getMetric().getMetricSections().size() < 
				map2.getMetric().getMetricSections().size())
			return -1;
		else return 0;
	}

	private int compareActive(Mapping map1, Mapping map2) {
		if(map1.getMappingState().ordinal() > map2.getMappingState().ordinal())
			return 1;
		else if(map1.getMappingState().ordinal() < map2.getMappingState().ordinal())
			return -1;
		else return 0;
	}

	private int compareMapping(Mapping map1, Mapping map2) {
		int result = map1.getName().compareTo(map2.getName());
		return result;
	}
	
	private int compareMetric(Mapping map1, Mapping map2) {
		int result = map1.getMetric().getName().compareTo(map2.getMetric().getName());
		return result;
	}
	
	private int compareDisplay(Mapping map1, Mapping map2) {
		int result = map1.getAugmentation().getName().compareTo(map2.getAugmentation().getName());
		return result;
	}

	/**
	 * Returns the sort criteria of this this sorter.
	 *
	 * @return the sort criterion
	 */
	public int getCriteria() {
		return criteria;
	}
}
