package metricsOverlay.views.toolbar;

import java.util.Set;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowPulldownDelegate;
import org.eclipse.ui.PlatformUI;

/**
 * This class provides the pop up mapping selection menu.
 * 
 * @author Matthew Harward
 * @version 19/8/2009
 */
public class AddElementHandler implements IWorkbenchWindowPulldownDelegate {

	private Button addButton;
	private Text text;
	private List list;
	private Button cancelButton;

	@Override
	public Menu getMenu(Control parent) {
		return null;
	}

	@Override
	public void dispose() {
	}

	@Override
	public void init(IWorkbenchWindow window) {
	}

	@Override
	public void run(IAction action) {
		Shell main = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getShell();
		final Shell shell = new Shell(main.getDisplay());
		shell.setText("Add Mappings");
		shell.setSize(300, 200);
		GridLayout gLayout = new GridLayout();
		gLayout.numColumns = 2;
		gLayout.makeColumnsEqualWidth = true;
		shell.setLayout(gLayout);

		list = new List(shell, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		GridData dat = new GridData(SWT.FILL, SWT.BEGINNING, true, true);
		dat.horizontalSpan = 2;
		list.setLayoutData(dat);
		MetricFile mf = MetricFileHandler.getInstance().getCurrentMFile();
		if (mf != null) {

			for (Mapping m : mf.getMetricDocument().getAllMappings()) {
				// if(m.getMappingState().equals(MappingState.UndeclaredMapping)){
				if (m.getMappingState().ordinal() > 0) { // <<-- Testing only
					list.add(m.getName());
				}
			}
		}

		text = new Text(shell, SWT.NONE);
		text.setEditable(false);
		text.setText("No mappings selected");
		if (list.getItemCount() <= 0) {
			text.setText("No mappings avaliable");
		}
		GridData dat2 = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
		dat2.horizontalSpan = 2;
		text.setLayoutData(dat2);

		addButton = new Button(shell, SWT.PUSH);
		addButton.setText("Add Mappings");
		addButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false,
				false));
		addButton.setEnabled(false);
		addButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// Add mappings to table, make them active
				MetricFile mf = MetricFileHandler.getInstance()
						.getCurrentMFile();
				Set<Mapping> maps = mf.getMetricDocument()
						.getAllMappings();
				for (String str : list.getSelection()) {
					for (Mapping m : maps) {
						if (m.getName().equals(str))
							m.setMappingState(MappingState.ActiveMapping);
					}
				}
				// May need to force an update here...

				// Then remove this window
				shell.dispose();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		cancelButton = new Button(shell, SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, false,
				false));
		cancelButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				shell.dispose();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		list.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				handleSelection();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				handleSelection();
			}
		});

		shell.open();

	}

	/**
	 * 
	 */
	private void handleSelection() {
		int i = list.getSelectionCount();
		if (i > 0) {
			text.setText(i + " mappings selected");
			addButton.setEnabled(true);
		} else {
			text.setText("No mappings selected");
			addButton.setEnabled(false);
		}
		if (i == 1) // Get the tense right!
			text.setText(i + " mapping selected");
	}

	@Override
	public void selectionChanged(IAction action, ISelection selection) {
	}
	/*
	 * public AddElementHandler() {
	 * System.out.println("Add Element Constructor"); }
	 * 
	 * @Override public void createContributionItems(IServiceLocator
	 * serviceLocator, IContributionRoot additions) { CommandContributionItem
	 * item = new CommandContributionItem( new CommandContributionItemParameter(
	 * PlatformUI.getWorkbench().getActiveWorkbenchWindow() , null, "someid",
	 * CommandContributionItem.STYLE_PUSH));
	 * 
	 * additions.addContributionItem(item, null); IMenuService menuService =
	 * (IMenuService) PlatformUI.getWorkbench() .getService(IMenuService.class);
	 * menuService.addContributionFactory(this); //
	 * /menuService.populateContributionManager(mgr, location)
	 * System.out.println("CreateCont");
	 * 
	 * }
	 */
}
