package metricsprovider;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import metricsOverlay.api.IMetricOverlay;
import metricsOverlay.api.MetricOverlayFactory;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingProperties;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.IntegerRange;

import org.eclipse.jface.text.IDocument;

import editorListener.handlers.UpdatedEditor;

/**
 * This class implements a basic exemplar metric
 * that provides a incrementing value for each line of
 * the text.
 * 
 * @author Matthew Harward
 * @version 9/8/09
 */
@Deprecated
public class LOCGenerator extends MetricsGenerator {
	
	/**
	 * @return a singleton instance of this class.
	 */
	public static LOCGenerator getInstance() {
		if(mGen == null)
			mGen = new LOCGenerator();
		return (LOCGenerator) mGen;
	}
	
	/**
	 * Main constructor, must call super.
	 */
	public LOCGenerator() {
		super();
	}
	
	/**
	 * This class creates a metric and combines it with existing editor
	 * information into the form of a metric file and then updates
	 * the observer with the MetricFile.
	 */
	public void generateMetrics(UpdatedEditor upEditor) {
		MetricFile mf = null;
		IMetricOverlay mOverlay = MetricOverlayFactory.instance();
		MetricDocument mDoc = mOverlay.createMetricDocument(upEditor.getPath());
		mf = new MetricFile(upEditor.getDocument(),upEditor.getWorkbenchPart(),mDoc,
				upEditor.getPath(),true);
		//createMapping(mDoc, upEditor.getDocument(), AugmentationName.COLOURCHIPS);
		//createMapping2(mDoc, upEditor.getDocument(), AugmentationName.TWOCOLOURING);
		//createMapping3(mDoc, upEditor.getDocument(), AugmentationName.UNERLINESTRATEGY);
		// createMapping(mDoc, iDoc, AugmentationName.ONECOLOURING);
		// createMapping(mDoc, iDoc, AugmentationName.COLOURBAR);
		setChanged();
		notifyObservers(mf);
	}
	
	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping(MetricDocument mDoc, IDocument iDoc,
			String displayType) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		Map<String, String> props = new HashMap<String, String>();
		props.put("Color1", "{250,20,20}");
		props.put("Color2", "{20,20,250}");
		//props.put(MappingProperties.ONLEFT, "false");
		props.put(MappingProperties.SHAPE, "downtri");
		Mapping map = new Mapping("LOC Mapping2", displayType, props);
		//mm.setMapping(map);

		Set<MetricSection> mSections = new HashSet<MetricSection>();
		for (int i = 0; i < lines; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				if (lineLength < 0)
					lineLength = 0;
				ms = new MetricSection(i, 0, i, lineLength, Integer.toString(i));
				ms.setAnnotation("New Metric Info For Line: " + i);
				mSections.add(ms);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Current Line2", new IntegerRange(0, lines));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}

	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping2(MetricDocument mDoc, IDocument iDoc,
			String displayType) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		Map<String, String> props = new HashMap<String, String>();
		props.put("Color1", "{140,250,250}");
		props.put("Color2", "{250,140,140}");
		Mapping map = new Mapping("LOC Mapping", displayType, props);
		map.setMetric(new ActualMetric("Current Line", new IntegerRange(0, lines)));
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		//mm.setMapping(map);
		for (int i = 0; i < lines; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				if (lineLength < 0)
					lineLength = 0;
				ms = new MetricSection(i, 0, i, lineLength, Integer.toString(i));
				mSections.add(ms);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Current Line3", new IntegerRange(0, lines));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}

	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping3(MetricDocument mDoc, IDocument iDoc,
			String displayType) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		Map<String, String> props = new HashMap<String, String>();
		props.put("Color1", "{255,250,170}");
		props.put("Color2", "{20,250,150}");
		//props.put(MappingProperties.ONLEFT, "false");
		//props.put(MappingProperties.SHAPE, "doubleleftarrow");
		props.put(MappingProperties.UNDERLINETYPE, MappingProperties.ULINE_SQUIGGLE);
		Mapping map = new Mapping("LOC Mapping3", displayType, props);
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		//mm.setMapping(map);
		for (int i = 0; i < lines; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				if (lineLength < 0)
					lineLength = 0;
				ms = new MetricSection(i, 0, i, lineLength, Integer.toString(i));
				mSections.add(ms);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Current Line4", new IntegerRange(0, lines));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}
	
}
