package metricsprovider;

import java.util.HashSet;
import java.util.Set;

import metricsOverlay.api.IMetricOverlay;
import metricsOverlay.api.MetricOverlayFactory;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.BackgroundLineAugmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.other.ShapeDirection;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.IntegerRange;

import org.eclipse.jface.text.IDocument;
import org.eclipse.swt.graphics.Color;

import editorListener.handlers.UpdatedEditor;

public class LineLengthGenerator extends MetricsGenerator {


	/**
	 * @return a singleton instance of this class.
	 */
	public static LineLengthGenerator getInstance() {
		if(mGen == null)
			mGen = new LineLengthGenerator();
		return (LineLengthGenerator) mGen;
	}
	
	/**
	 * Main constructor, must call super.
	 */
	public LineLengthGenerator() {
		super();
	}
	
	/**
	 * This class creates a metric and combines it with existing editor
	 * information into the form of a metric file and then updates
	 * the observer with the MetricFile.
	 */
	public void generateMetrics(UpdatedEditor upEditor) {
		System.out.println("Generate");
		MetricFile mf = null;
		IMetricOverlay mOverlay = MetricOverlayFactory.instance();
		MetricDocument mDoc = mOverlay.createMetricDocument(upEditor.getPath());
		mf = new MetricFile(upEditor.getDocument(),upEditor.getWorkbenchPart(),mDoc,
				upEditor.getPath(),true);
		createMapping(mDoc, upEditor.getDocument());
		createMapping2(mDoc, upEditor.getDocument());
		// createMapping(mDoc, iDoc, AugmentationName.ONECOLOURING);
		// createMapping(mDoc, iDoc, AugmentationName.COLOURBAR);
		setChanged();
		notifyObservers(mf);
	}
	
	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping(MetricDocument mDoc, IDocument iDoc) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		//Map<String, String> props = new HashMap<String, String>();
		//props.put("Color1", "{250,250,180}");
		//props.put("Color2", "{20,20,250}");
		Mapping map = new Mapping("Line Length Map", false);
		BackgroundLineAugmentation ma = new BackgroundLineAugmentation("A line augmentation");
		ma.addColor(new Color(null, 255,255,255));
		ma.addColor(new Color(null,0,200,0));
		map.setAugmentation(ma);
		//mm.setMapping(map);
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		int linesize = 0;
		for (int i = 0; i < lines; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				if (lineLength < 0)
					lineLength = 0;
				if(lineLength > 0)
				{
					ms = new MetricSection(i, 0, i, lineLength, Integer.toString(lineLength));
					ms.setAnnotation("New Metric Info For Line: " + i);
					mSections.add(ms);
				}
				if(lineLength > linesize)
					linesize = lineLength;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Current Line2", new IntegerRange(0, linesize));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}
	
	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping2(MetricDocument mDoc, IDocument iDoc) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		//Map<String, String> props = new HashMap<String, String>();
		//props.put("Color1", "{20,250,20}");
		//props.put("Color2", "{240,20,20}");
		//props.put(MappingProperties.SHAPE, ShapeManager.DOUBLERIGHTARROW);
		Mapping map = new Mapping("LOC Mapping2", false);
		MarginAugmentation ma = new MarginAugmentation("A margin augmentation");
		ma.addColor(new Color(null, 20,250,20));
		ma.addColor(new Color(null,240,20,20));
		ma.setShape(ShapeType.doublearrow, ShapeDirection.up);
		ma.setIsLeft(true);
		ma.setDiscrete(true);
		map.setAugmentation(ma);
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		//mm.setMapping(map);
		for (int i = 0; i < lines - 1; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				int lineVal = 0;
				if(iDoc.get(iDoc.getLineOffset(i),iDoc.getLineLength(i)-1).contains("}"))
						lineVal = 1;
				if(iDoc.get(iDoc.getLineOffset(i),iDoc.getLineLength(i)-1).contains("{"))
						lineVal = 2;
				if (lineLength < 0)
					lineLength = 0;
				if(lineVal > 0){
					ms = new MetricSection(i, 0, i, lineLength, Integer.toString(lineVal));
					ms.setAnnotation("New Metric Info For Line: " + i);
					mSections.add(ms);
				}
			} catch (Exception e) {
				System.out.println("On line:" + i);
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Current Line3", new IntegerRange(1,2));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}
	
}
