package metricsprovider;

import java.util.Observable;
import java.util.Observer;

import editorListener.handlers.EditorListener;
import editorListener.handlers.UpdatedEditor;

/**
 * This class is an static observer stub designed to listen to the
 * EditorListener plugin. It provides update information to the appropriate
 * MetricGenerator subclass.
 * 
 * @author Matthew Harward
 * @version 9/8/2009
 */
public class ListenerObserver implements Observer {

	//Singleton
	protected static ListenerObserver obs;
	
	/**
	 * Constructor, attaches itself as an observer to the
	 * EditorListener plugin.
	 */
	protected ListenerObserver() {
		EditorListener.getInstance().addObserver(this);
	}
	
	/**
	 * @return A singleton instance of this class.
	 */
	public static ListenerObserver getInstance() {
		if(obs == null)
			obs = new ListenerObserver();
		return obs;
	}

	/**
	 * This method gets called when EditorListener updates,
	 * in turn, it calls an update for the appropriate MetricsGenerator.
	 */
	@Override
	public void update(Observable arg0, Object arg1) {
		if(arg1 instanceof UpdatedEditor)
			//LineLengthGenerator.getInstance().generateMetrics((UpdatedEditor) arg1);
			//TeamPatchwork.getInstance().generateMetrics((UpdatedEditor) arg1);
			PaperExamples.getInstance().generateMetrics((UpdatedEditor) arg1);
	}
}
