package metricsprovider;

import java.util.Observable;
import metricsOverlay.api.MetricOverlayFactory;
import editorListener.handlers.UpdatedEditor;

/**
 * This abstract class provides basic functionality for adding 
 * metrics information to the overlay.
 * 
 * @author Matthew Harward
 * @version 9/8/2009
 */
public abstract class MetricsGenerator extends Observable {

	protected static MetricsGenerator mGen;
	
	/**
	 * Default constructor, adds the MetricOverlay as a listener.
	 */
	protected MetricsGenerator() {
		addObserver(MetricOverlayFactory.instance());
	}
	
	/**
	 * Default singleton method, should be overriden.
	 */
	public static MetricsGenerator getInstance(){
		return mGen;
	}

	/**
	 * This method is the main metric generation path. Subclasses
	 * should override this method and make sure it sets the observer
	 * as being changed and notifies the observer with a either a MetricFile
	 * or an UpdatedEditor if XML is provided.
	 * 
	 * @param upEditor the editor information for the current update.
	 */
	public abstract void generateMetrics(UpdatedEditor upEditor);

}
