package metricsprovider;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.swt.graphics.Color;

import editorListener.handlers.UpdatedEditor;
import metricsOverlay.api.IMetricOverlay;
import metricsOverlay.api.MetricOverlayFactory;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.BackgroundLineAugmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.UnderlineAugmentation;
import metricsOverlay.model.augmentation.other.ShapeDirection;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.augmentation.other.UnderlineType;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.OrderedNominalRange;

public class PaperExamples extends MetricsGenerator {

	/**
	 * @return a singleton instance of this class.
	 */
	public static PaperExamples getInstance() {
		if(mGen == null)
			mGen = new PaperExamples();
		return (PaperExamples) mGen;
	}
	
	/**
	 * Main constructor, must call super.
	 */
	public PaperExamples() {
		super();
	}
	
	@Override
	public void generateMetrics(UpdatedEditor upEditor) {
		MetricFile mf = null;
		MetricDocument mDoc = createModel(upEditor);
		mf = new MetricFile(upEditor.getDocument(),upEditor.getWorkbenchPart(),mDoc,
				upEditor.getPath(),true);
		setChanged();
		notifyObservers(mf);
		
	}

	private MetricDocument createModel(UpdatedEditor upEditor) {
		IPath iPath = upEditor.getPath();
		IMetricOverlay mOverlay = MetricOverlayFactory.instance();
		MetricDocument mDoc = mOverlay.createMetricDocument(iPath);
		
		//CODE AGE METRIC
		
		//Granularity by days...
		IntegerRange ir = new IntegerRange(0,200);
		ActualMetric codeAge = new ActualMetric("CodeAge", ir);
		Set<MetricSection> age = new HashSet<MetricSection>();
		try {
			for(int i = 0; i < upEditor.getDocument().getNumberOfLines(); i++) {
				int val = 199;
				if(i >= 29 && i < 33) { val = 0; }
				if(i >= 35 && i < 37) { val = 30; }
			MetricSection ms = new MetricSection(i, 0, i, 
					upEditor.getDocument().getLineLength(i) - 1, String.valueOf(val));
			ms.setAnnotation("This code was last altered 4 days ago");
			age.add(ms);
			}
		} catch (Exception e) {}
		codeAge.setMetricSections(age);
		
		//You can just directly add a metric, but we will create a basic mapping
		//and augmentation to go with it.
		BackgroundLineAugmentation bgLineAug = 
			new BackgroundLineAugmentation("Yellow BG Colouring");
		bgLineAug.setEditable(false);
		bgLineAug.setAnnotations(true);
		bgLineAug.setImportance(60);
		List<Color> cols = new ArrayList<Color>();
		cols.add(new Color(null, 255,255,150));
		cols.add(new Color(null, 255,255,255));
		bgLineAug.setColors(cols);
		
		Mapping mapAge = new Mapping("Visualising CodeAge");
		mapAge.setAugmentation(bgLineAug);
		mapAge.setMetric(codeAge);
		mapAge.setEditable(false);
		mapAge.setMappingState(MappingState.ActiveMapping);
		
		//Add mapping into doc, will add metric and aug too...
		mDoc.addMapping(mapAge);
		
		//DEVELOPER METRIC
		
		Map<Integer,String> developers = new HashMap<Integer,String>();
		//developers.put(1, "Paul");
		//developers.put(2, "Peter");
		//developers.put(3, "Patricia");
		//OrderedNominalRange onr = new OrderedNominalRange(developers);
		IntegerRange intRang = new IntegerRange(1,3);
		ActualMetric devMet = new ActualMetric("EditedBy", intRang);
		Set<MetricSection> devBy = new HashSet<MetricSection>();
		try {
			for(int j = 0; j < 50 ; j++){
				if(j >= 29 && j < 33) { 
					MetricSection ms = new MetricSection(j, 2, j, 22, "1");
					ms.setAnnotation("Paul Wrote This!");
					devBy.add(ms);
				}
				else if(j >= 35 && j < 37) {
					MetricSection ms = new MetricSection(j, 2, j, 22, "2");
					ms.setAnnotation("Paul corrected a bug\nSee Tracker: #26573");
					devBy.add(ms);
					
				}
				else {
					MetricSection ms = new MetricSection(j, 2, j, 22, "3");
					ms.setAnnotation("Paul Fixed A Bug Here,\nSee Tracker: #26573");
					devBy.add(ms);
					
				}
			}
		} catch (Exception e) {}
		devMet.setMetricSections(devBy);
		
		//You can just directly add a metric, but we will create a basic mapping
		//and augmentation to go with it.
		MarginAugmentation margAug = 
			new MarginAugmentation("Dev Margin Chips");
		margAug.setEditable(false);
		margAug.setAnnotations(true);
		margAug.setImportance(40);
		margAug.setIsLeft(true);
		margAug.setDiscrete(true);
		margAug.setShape(ShapeType.roundedsquare, ShapeDirection.left);
		List<Color> cols2 = new ArrayList<Color>();
		cols2.add(new Color(null, 0,0,255));
		cols2.add(new Color(null, 240,0,0));
		cols2.add(new Color(null, 255,170,100));
		margAug.setColors(cols2);
		
		Mapping mapDev = new Mapping("Last Edited By Visuals");
		mapDev.setAugmentation(margAug);
		mapDev.setMetric(devMet);
		mapDev.setEditable(false);
		mapDev.setMappingState(MappingState.ActiveMapping);
		
		//Add the mapping into the document
		mDoc.addMapping(mapDev);
		
		createMapping(mDoc, upEditor.getDocument());
		createMapping2(mDoc, upEditor.getDocument());
		underlineMapping(mDoc, upEditor.getDocument());
		
		return mDoc;
	}
	

	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping(MetricDocument mDoc, IDocument iDoc) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		//Map<String, String> props = new HashMap<String, String>();
		//props.put("Color1", "{250,250,180}");
		//props.put("Color2", "{20,20,250}");
		Mapping map = new Mapping("Line Length Mapping", false);
		BackgroundLineAugmentation ma = new BackgroundLineAugmentation("Green Line Length Augmentation");
		ma.addColor(new Color(null, 255,255,255));
		ma.addColor(new Color(null,0,200,0));
		map.setAugmentation(ma);
		//mm.setMapping(map);
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		int linesize = 0;
		for (int i = 0; i < lines; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				if (lineLength < 0)
					lineLength = 0;
				if(lineLength > 0)
				{
					ms = new MetricSection(i, 0, i, lineLength, Integer.toString(lineLength));
					ms.setAnnotation("New Metric Info For Line: " + i);
					mSections.add(ms);
				}
				if(lineLength > linesize)
					linesize = lineLength;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Line Length", new IntegerRange(0, linesize));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}
	
	/**
	 * @param mDoc
	 * @param iDoc
	 */
	private void createMapping2(MetricDocument mDoc, IDocument iDoc) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		//Map<String, String> props = new HashMap<String, String>();
		//props.put("Color1", "{20,250,20}");
		//props.put("Color2", "{240,20,20}");
		//props.put(MappingProperties.SHAPE, ShapeManager.DOUBLERIGHTARROW);
		Mapping map = new Mapping("Code Block Visualisation", false);
		MarginAugmentation ma = new MarginAugmentation("Right Margin Pointers");
		ma.addColor(new Color(null, 20,250,20));
		ma.addColor(new Color(null,240,20,20));
		ma.setShape(ShapeType.doublearrow, ShapeDirection.left);
		ma.setIsLeft(false);
		ma.setDiscrete(true);
		map.setAugmentation(ma);
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		//mm.setMapping(map);
		for (int i = 0; i < lines - 1; i++) {
			MetricSection ms;
			try {
				int lineLength = iDoc.getLineLength(i) - 1;
				int lineVal = 0;
				if(iDoc.get(iDoc.getLineOffset(i),iDoc.getLineLength(i)-1).contains("}"))
						lineVal = 1;
				if(iDoc.get(iDoc.getLineOffset(i),iDoc.getLineLength(i)-1).contains("{"))
						lineVal = 2;
				if (lineLength < 0)
					lineLength = 0;
				if(lineVal > 0){
					ms = new MetricSection(i, 0, i, lineLength, Integer.toString(lineVal));
					ms.setAnnotation("New Metric Info For Line: " + i);
					mSections.add(ms);
				}
			} catch (Exception e) {
				System.out.println("On line:" + i);
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Open/closing codeblock", new IntegerRange(1,2));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}
	
	private void underlineMapping(MetricDocument mDoc, IDocument iDoc) {
		//MappedMetric mm = new MappedMetric();
		int lines = iDoc.getNumberOfLines();
		//Map<String, String> props = new HashMap<String, String>();
		//props.put("Color1", "{250,250,180}");
		//props.put("Color2", "{20,20,250}");
		Mapping map = new Mapping("Primitive Mapping", false);
		UnderlineAugmentation ma = new UnderlineAugmentation("Swiggle Underline");
		ma.setUnderlineType(UnderlineType.SwiggleUnderline);
		ma.addColor(new Color(null, 150,150,255));
		map.setAugmentation(ma);
		//mm.setMapping(map);
		Set<MetricSection> mSections = new HashSet<MetricSection>();
		for(int i = 0; i < iDoc.getNumberOfLines(); i++) {
			try {
				int start = iDoc.getLineOffset(i);
				int length = iDoc.getLineLength(i);
				String curLine = iDoc.get(start, length);
				if(curLine.contains("int")) {
					int pos = curLine.indexOf("int");
					MetricSection m = new MetricSection(i, pos, i, pos+3, "1");
					mSections.add(m);
				}
			} catch (BadLocationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		ActualMetric met = new ActualMetric("Primitive Detector", new IntegerRange(0, 1));
		met.setMetricSections(mSections);
		map.setMetric(met);
		mDoc.addMapping(map);
	}
	
	
}
