package metricsprovider;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IPath;
import org.eclipse.swt.graphics.Color;

import editorListener.handlers.UpdatedEditor;
import metricsOverlay.api.IMetricOverlay;
import metricsOverlay.api.MetricOverlayFactory;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.BackgroundLineAugmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.other.ShapeDirection;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.OrderedNominalRange;

public class TeamPatchwork extends MetricsGenerator {

	/**
	 * @return a singleton instance of this class.
	 */
	public static TeamPatchwork getInstance() {
		if(mGen == null)
			mGen = new TeamPatchwork();
		return (TeamPatchwork) mGen;
	}
	
	/**
	 * Main constructor, must call super.
	 */
	public TeamPatchwork() {
		super();
	}
	
	@Override
	public void generateMetrics(UpdatedEditor upEditor) {
		MetricFile mf = null;
		MetricDocument mDoc = createModel(upEditor.getPath());
		mf = new MetricFile(upEditor.getDocument(),upEditor.getWorkbenchPart(),mDoc,
				upEditor.getPath(),true);
		setChanged();
		notifyObservers(mf);
		
	}

	private MetricDocument createModel(IPath iPath) {
		IMetricOverlay mOverlay = MetricOverlayFactory.instance();
		MetricDocument mDoc = mOverlay.createMetricDocument(iPath);
		
		//CODE AGE METRIC
		
		//Granularity by days...
		IntegerRange ir = new IntegerRange(0,1000);
		ActualMetric codeAge = new ActualMetric("CodeAge Metric", ir);
		Set<MetricSection> age = new HashSet<MetricSection>();
		try {
			MetricSection ms = new MetricSection(0, 0, 10, 22, "500");
			ms.setAnnotation("This code was written 2 days ago");
			age.add(ms);
		} catch (Exception e) {}
		codeAge.setMetricSections(age);
		
		//You can just directly add a metric, but we will create a basic mapping
		//and augmentation to go with it.
		BackgroundLineAugmentation bgLineAug = 
			new BackgroundLineAugmentation("Age Colouring");
		bgLineAug.setEditable(false);
		bgLineAug.setAnnotations(true);
		bgLineAug.setImportance(60);
		List<Color> cols = new ArrayList<Color>();
		cols.add(new Color(null, 180,180,255));
		cols.add(new Color(null, 255,255,255));
		bgLineAug.setColors(cols);
		
		Mapping mapAge = new Mapping("Code Age Visuals");
		mapAge.setAugmentation(bgLineAug);
		mapAge.setMetric(codeAge);
		mapAge.setEditable(false);
		mapAge.setMappingState(MappingState.ActiveMapping);
		
		//Add mapping into doc, will add metric and aug too...
		mDoc.addMapping(mapAge);
		
		//DEVELOPER METRIC
		
		Map<Integer,String> developers = new HashMap<Integer,String>();
		developers.put(1, "Paul");
		developers.put(2, "Peter");
		developers.put(3, "Patricia");
		OrderedNominalRange onr = new OrderedNominalRange(developers);
		ActualMetric devMet = new ActualMetric("EditedBy Metric", onr);
		Set<MetricSection> devBy = new HashSet<MetricSection>();
		try {
			MetricSection ms = new MetricSection(8, 2, 10, 22, "1");
			ms.setAnnotation("Paul Wrote This!");
			devBy.add(ms);
		} catch (Exception e) {}
		devMet.setMetricSections(devBy);
		
		//You can just directly add a metric, but we will create a basic mapping
		//and augmentation to go with it.
		MarginAugmentation margAug = 
			new MarginAugmentation("Dev Margin Chips");
		margAug.setEditable(false);
		margAug.setAnnotations(true);
		margAug.setImportance(40);
		margAug.setIsLeft(false);
		margAug.setShape(ShapeType.roundedsquare, ShapeDirection.left);
		List<Color> cols2 = new ArrayList<Color>();
		cols2.add(new Color(null, 180,180,255));
		cols2.add(new Color(null, 255,180,180));
		cols2.add(new Color(null, 180,255,180));
		margAug.setColors(cols2);
		
		Mapping mapDev = new Mapping("Last Edited By Visuals");
		mapDev.setAugmentation(margAug);
		mapDev.setMetric(devMet);
		mapDev.setEditable(false);
		mapDev.setMappingState(MappingState.ActiveMapping);
		
		//Add the mapping into the document
		mDoc.addMapping(mapDev);
		
		return mDoc;
	}
	
}
